# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blob.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nblob.proto\x12\x06gitaly\x1a\nlint.proto\x1a\x0cshared.proto\"Z\n\x0eGetBlobRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0b\n\x03oid\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\x03\":\n\x0fGetBlobResponse\x12\x0c\n\x04size\x18\x01 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x0b\n\x03oid\x18\x03 \x01(\t\"\xbc\x01\n\x0fGetBlobsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12<\n\x0erevision_paths\x18\x02 \x03(\x0b\x32$.gitaly.GetBlobsRequest.RevisionPath\x12\r\n\x05limit\x18\x03 \x01(\x03\x1a.\n\x0cRevisionPath\x12\x10\n\x08revision\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\x0c\"\xa1\x01\n\x10GetBlobsResponse\x12\x0c\n\x04size\x18\x01 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x0b\n\x03oid\x18\x03 \x01(\t\x12\x14\n\x0cis_submodule\x18\x04 \x01(\x08\x12\x0c\n\x04mode\x18\x05 \x01(\x05\x12\x10\n\x08revision\x18\x06 \x01(\t\x12\x0c\n\x04path\x18\x07 \x01(\x0c\x12 \n\x04type\x18\x08 \x01(\x0e\x32\x12.gitaly.ObjectType\"\x8b\x01\n\x10ListBlobsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\trevisions\x18\x02 \x03(\t\x12\r\n\x05limit\x18\x03 \x01(\r\x12\x13\n\x0b\x62ytes_limit\x18\x04 \x01(\x03\x12\x12\n\nwith_paths\x18\x05 \x01(\x08\"\x81\x01\n\x11ListBlobsResponse\x12-\n\x05\x62lobs\x18\x01 \x03(\x0b\x32\x1e.gitaly.ListBlobsResponse.Blob\x1a=\n\x04\x42lob\x12\x0b\n\x03oid\x18\x01 \x01(\t\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x12\x0c\n\x04path\x18\x04 \x01(\x0c\"g\n\x13ListAllBlobsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\r\n\x05limit\x18\x02 \x01(\r\x12\x13\n\x0b\x62ytes_limit\x18\x03 \x01(\x03\"y\n\x14ListAllBlobsResponse\x12\x30\n\x05\x62lobs\x18\x01 \x03(\x0b\x32!.gitaly.ListAllBlobsResponse.Blob\x1a/\n\x04\x42lob\x12\x0b\n\x03oid\x18\x01 \x01(\t\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\"Z\n\nLFSPointer\x12\x0c\n\x04size\x18\x01 \x01(\x03\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x0b\n\x03oid\x18\x03 \x01(\t\x12\x11\n\tfile_size\x18\x04 \x01(\x03\x12\x10\n\x08\x66ile_oid\x18\x05 \x01(\x0c\"W\n\x15GetLFSPointersRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08\x62lob_ids\x18\x02 \x03(\t\"B\n\x16GetLFSPointersResponse\x12(\n\x0clfs_pointers\x18\x01 \x03(\x0b\x32\x12.gitaly.LFSPointer\"h\n\x16ListLFSPointersRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\trevisions\x18\x02 \x03(\t\x12\r\n\x05limit\x18\x03 \x01(\x05\"C\n\x17ListLFSPointersResponse\x12(\n\x0clfs_pointers\x18\x01 \x03(\x0b\x32\x12.gitaly.LFSPointer\"X\n\x19ListAllLFSPointersRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\r\n\x05limit\x18\x03 \x01(\x05\"F\n\x1aListAllLFSPointersResponse\x12(\n\x0clfs_pointers\x18\x01 \x03(\x0b\x32\x12.gitaly.LFSPointer2\xdd\x04\n\x0b\x42lobService\x12\x44\n\x07GetBlob\x12\x16.gitaly.GetBlobRequest\x1a\x17.gitaly.GetBlobResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12G\n\x08GetBlobs\x12\x17.gitaly.GetBlobsRequest\x1a\x18.gitaly.GetBlobsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12J\n\tListBlobs\x12\x18.gitaly.ListBlobsRequest\x1a\x19.gitaly.ListBlobsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12S\n\x0cListAllBlobs\x12\x1b.gitaly.ListAllBlobsRequest\x1a\x1c.gitaly.ListAllBlobsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12Y\n\x0eGetLFSPointers\x12\x1d.gitaly.GetLFSPointersRequest\x1a\x1e.gitaly.GetLFSPointersResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\\\n\x0fListLFSPointers\x12\x1e.gitaly.ListLFSPointersRequest\x1a\x1f.gitaly.ListLFSPointersResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\x65\n\x12ListAllLFSPointers\x12!.gitaly.ListAllLFSPointersRequest\x1a\".gitaly.ListAllLFSPointersResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'blob_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _GETBLOBREQUEST.fields_by_name['repository']._options = None
  _GETBLOBREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETBLOBSREQUEST.fields_by_name['repository']._options = None
  _GETBLOBSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTBLOBSREQUEST.fields_by_name['repository']._options = None
  _LISTBLOBSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTALLBLOBSREQUEST.fields_by_name['repository']._options = None
  _LISTALLBLOBSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETLFSPOINTERSREQUEST.fields_by_name['repository']._options = None
  _GETLFSPOINTERSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTLFSPOINTERSREQUEST.fields_by_name['repository']._options = None
  _LISTLFSPOINTERSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTALLLFSPOINTERSREQUEST.fields_by_name['repository']._options = None
  _LISTALLLFSPOINTERSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _BLOBSERVICE.methods_by_name['GetBlob']._options = None
  _BLOBSERVICE.methods_by_name['GetBlob']._serialized_options = b'\372\227(\002\010\002'
  _BLOBSERVICE.methods_by_name['GetBlobs']._options = None
  _BLOBSERVICE.methods_by_name['GetBlobs']._serialized_options = b'\372\227(\002\010\002'
  _BLOBSERVICE.methods_by_name['ListBlobs']._options = None
  _BLOBSERVICE.methods_by_name['ListBlobs']._serialized_options = b'\372\227(\002\010\002'
  _BLOBSERVICE.methods_by_name['ListAllBlobs']._options = None
  _BLOBSERVICE.methods_by_name['ListAllBlobs']._serialized_options = b'\372\227(\002\010\002'
  _BLOBSERVICE.methods_by_name['GetLFSPointers']._options = None
  _BLOBSERVICE.methods_by_name['GetLFSPointers']._serialized_options = b'\372\227(\002\010\002'
  _BLOBSERVICE.methods_by_name['ListLFSPointers']._options = None
  _BLOBSERVICE.methods_by_name['ListLFSPointers']._serialized_options = b'\372\227(\002\010\002'
  _BLOBSERVICE.methods_by_name['ListAllLFSPointers']._options = None
  _BLOBSERVICE.methods_by_name['ListAllLFSPointers']._serialized_options = b'\372\227(\002\010\002'
  _globals['_GETBLOBREQUEST']._serialized_start=48
  _globals['_GETBLOBREQUEST']._serialized_end=138
  _globals['_GETBLOBRESPONSE']._serialized_start=140
  _globals['_GETBLOBRESPONSE']._serialized_end=198
  _globals['_GETBLOBSREQUEST']._serialized_start=201
  _globals['_GETBLOBSREQUEST']._serialized_end=389
  _globals['_GETBLOBSREQUEST_REVISIONPATH']._serialized_start=343
  _globals['_GETBLOBSREQUEST_REVISIONPATH']._serialized_end=389
  _globals['_GETBLOBSRESPONSE']._serialized_start=392
  _globals['_GETBLOBSRESPONSE']._serialized_end=553
  _globals['_LISTBLOBSREQUEST']._serialized_start=556
  _globals['_LISTBLOBSREQUEST']._serialized_end=695
  _globals['_LISTBLOBSRESPONSE']._serialized_start=698
  _globals['_LISTBLOBSRESPONSE']._serialized_end=827
  _globals['_LISTBLOBSRESPONSE_BLOB']._serialized_start=766
  _globals['_LISTBLOBSRESPONSE_BLOB']._serialized_end=827
  _globals['_LISTALLBLOBSREQUEST']._serialized_start=829
  _globals['_LISTALLBLOBSREQUEST']._serialized_end=932
  _globals['_LISTALLBLOBSRESPONSE']._serialized_start=934
  _globals['_LISTALLBLOBSRESPONSE']._serialized_end=1055
  _globals['_LISTALLBLOBSRESPONSE_BLOB']._serialized_start=766
  _globals['_LISTALLBLOBSRESPONSE_BLOB']._serialized_end=813
  _globals['_LFSPOINTER']._serialized_start=1057
  _globals['_LFSPOINTER']._serialized_end=1147
  _globals['_GETLFSPOINTERSREQUEST']._serialized_start=1149
  _globals['_GETLFSPOINTERSREQUEST']._serialized_end=1236
  _globals['_GETLFSPOINTERSRESPONSE']._serialized_start=1238
  _globals['_GETLFSPOINTERSRESPONSE']._serialized_end=1304
  _globals['_LISTLFSPOINTERSREQUEST']._serialized_start=1306
  _globals['_LISTLFSPOINTERSREQUEST']._serialized_end=1410
  _globals['_LISTLFSPOINTERSRESPONSE']._serialized_start=1412
  _globals['_LISTLFSPOINTERSRESPONSE']._serialized_end=1479
  _globals['_LISTALLLFSPOINTERSREQUEST']._serialized_start=1481
  _globals['_LISTALLLFSPOINTERSREQUEST']._serialized_end=1569
  _globals['_LISTALLLFSPOINTERSRESPONSE']._serialized_start=1571
  _globals['_LISTALLLFSPOINTERSRESPONSE']._serialized_end=1641
  _globals['_BLOBSERVICE']._serialized_start=1644
  _globals['_BLOBSERVICE']._serialized_end=2249
# @@protoc_insertion_point(module_scope)
