# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: operations.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import errors_pb2 as errors__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10operations.proto\x12\x06gitaly\x1a\x0c\x65rrors.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\nlint.proto\x1a\x0cshared.proto\"\x8d\x01\n\x17UserCreateBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x13\n\x0b\x62ranch_name\x18\x02 \x01(\x0c\x12\x1a\n\x04user\x18\x03 \x01(\x0b\x32\x0c.gitaly.User\x12\x13\n\x0bstart_point\x18\x04 \x01(\x0c\"S\n\x18UserCreateBranchResponse\x12\x1e\n\x06\x62ranch\x18\x01 \x01(\x0b\x32\x0e.gitaly.BranchJ\x04\x08\x02\x10\x03R\x11pre_receive_error\"P\n\x15UserCreateBranchError\x12.\n\x0b\x63ustom_hook\x18\x01 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x42\x07\n\x05\x65rror\"\x98\x01\n\x17UserUpdateBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x13\n\x0b\x62ranch_name\x18\x02 \x01(\x0c\x12\x1a\n\x04user\x18\x03 \x01(\x0b\x32\x0c.gitaly.User\x12\x0e\n\x06newrev\x18\x04 \x01(\x0c\x12\x0e\n\x06oldrev\x18\x05 \x01(\x0c\"5\n\x18UserUpdateBranchResponse\x12\x19\n\x11pre_receive_error\x18\x01 \x01(\t\"\x92\x01\n\x17UserDeleteBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x13\n\x0b\x62ranch_name\x18\x02 \x01(\x0c\x12\x1a\n\x04user\x18\x03 \x01(\x0b\x32\x0c.gitaly.User\x12\x18\n\x10\x65xpected_old_oid\x18\x04 \x01(\t\"3\n\x18UserDeleteBranchResponseJ\x04\x08\x01\x10\x02R\x11pre_receive_error\"\xbc\x01\n\x15UserDeleteBranchError\x12\x30\n\x0c\x61\x63\x63\x65ss_check\x18\x01 \x01(\x0b\x32\x18.gitaly.AccessCheckErrorH\x00\x12\x38\n\x10reference_update\x18\x02 \x01(\x0b\x32\x1c.gitaly.ReferenceUpdateErrorH\x00\x12.\n\x0b\x63ustom_hook\x18\x03 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x42\x07\n\x05\x65rror\"\x8c\x01\n\x14UserDeleteTagRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08tag_name\x18\x02 \x01(\x0c\x12\x1a\n\x04user\x18\x03 \x01(\x0b\x32\x0c.gitaly.User\x12\x18\n\x10\x65xpected_old_oid\x18\x04 \x01(\t\"2\n\x15UserDeleteTagResponse\x12\x19\n\x11pre_receive_error\x18\x01 \x01(\t\"\xcb\x01\n\x14UserCreateTagRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08tag_name\x18\x02 \x01(\x0c\x12\x1a\n\x04user\x18\x03 \x01(\x0b\x32\x0c.gitaly.User\x12\x17\n\x0ftarget_revision\x18\x04 \x01(\x0c\x12\x0f\n\x07message\x18\x05 \x01(\x0c\x12-\n\ttimestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"X\n\x15UserCreateTagResponse\x12\x18\n\x03tag\x18\x01 \x01(\x0b\x32\x0b.gitaly.TagJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04R\x06\x65xistsR\x11pre_receive_error\"\xf3\x01\n\x12UserCreateTagError\x12\x30\n\x0c\x61\x63\x63\x65ss_check\x18\x01 \x01(\x0b\x32\x18.gitaly.AccessCheckErrorH\x00\x12\x38\n\x10reference_update\x18\x02 \x01(\x0b\x32\x1c.gitaly.ReferenceUpdateErrorH\x00\x12.\n\x0b\x63ustom_hook\x18\x03 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x12\x38\n\x10reference_exists\x18\x04 \x01(\x0b\x32\x1c.gitaly.ReferenceExistsErrorH\x00\x42\x07\n\x05\x65rror\"\xee\x01\n\x16UserMergeBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x11\n\tcommit_id\x18\x03 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\x0c\x12\x0f\n\x07message\x18\x05 \x01(\x0c\x12-\n\ttimestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\x08 \x01(\t\x12\r\n\x05\x61pply\x18\x06 \x01(\x08\"\x81\x01\n\x17UserMergeBranchResponse\x12\x11\n\tcommit_id\x18\x01 \x01(\t\x12\x34\n\rbranch_update\x18\x03 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdateJ\x04\x08\x02\x10\x03J\x04\x08\x04\x10\x05R\x11pre_receive_error\"\xf1\x01\n\x14UserMergeBranchError\x12\x30\n\x0c\x61\x63\x63\x65ss_check\x18\x01 \x01(\x0b\x32\x18.gitaly.AccessCheckErrorH\x00\x12\x38\n\x10reference_update\x18\x02 \x01(\x0b\x32\x1c.gitaly.ReferenceUpdateErrorH\x00\x12.\n\x0b\x63ustom_hook\x18\x03 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x12\x34\n\x0emerge_conflict\x18\x04 \x01(\x0b\x32\x1a.gitaly.MergeConflictErrorH\x00\x42\x07\n\x05\x65rror\"\xae\x02\n\x15UserMergeToRefRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x12\n\nsource_sha\x18\x03 \x01(\t\x12\x12\n\x06\x62ranch\x18\x04 \x01(\x0c\x42\x02\x18\x01\x12\x12\n\ntarget_ref\x18\x05 \x01(\x0c\x12\x0f\n\x07message\x18\x06 \x01(\x0c\x12\x18\n\x10\x66irst_parent_ref\x18\x07 \x01(\x0c\x12\x1b\n\x0f\x61llow_conflicts\x18\x08 \x01(\x08\x42\x02\x18\x01\x12-\n\ttimestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\n \x01(\t\"D\n\x16UserMergeToRefResponse\x12\x11\n\tcommit_id\x18\x01 \x01(\tJ\x04\x08\x02\x10\x03R\x11pre_receive_error\"\xed\x01\n\x16UserRebaseToRefRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x12\n\nsource_sha\x18\x03 \x01(\t\x12\x12\n\ntarget_ref\x18\x05 \x01(\x0c\x12\x18\n\x10\x66irst_parent_ref\x18\x07 \x01(\x0c\x12-\n\ttimestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\n \x01(\t\",\n\x17UserRebaseToRefResponse\x12\x11\n\tcommit_id\x18\x01 \x01(\t\"X\n\x15OperationBranchUpdate\x12\x11\n\tcommit_id\x18\x01 \x01(\t\x12\x14\n\x0crepo_created\x18\x02 \x01(\x08\x12\x16\n\x0e\x62ranch_created\x18\x03 \x01(\x08\"\x9c\x01\n\x13UserFFBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x11\n\tcommit_id\x18\x03 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\x0c\x12\x18\n\x10\x65xpected_old_oid\x18\x05 \x01(\t\"g\n\x14UserFFBranchResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdate\x12\x19\n\x11pre_receive_error\x18\x02 \x01(\t\"\x86\x03\n\x15UserCherryPickRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12!\n\x06\x63ommit\x18\x03 \x01(\x0b\x32\x11.gitaly.GitCommit\x12\x13\n\x0b\x62ranch_name\x18\x04 \x01(\x0c\x12\x0f\n\x07message\x18\x05 \x01(\x0c\x12\x19\n\x11start_branch_name\x18\x06 \x01(\x0c\x12,\n\x10start_repository\x18\x07 \x01(\x0b\x32\x12.gitaly.Repository\x12\x0f\n\x07\x64ry_run\x18\x08 \x01(\x08\x12-\n\ttimestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\n \x01(\t\x12\x1a\n\x12\x63ommit_author_name\x18\x0b \x01(\x0c\x12\x1b\n\x13\x63ommit_author_email\x18\x0c \x01(\x0c\"\xb2\x01\n\x16UserCherryPickResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdateJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06R\x11\x63reate_tree_errorR\x0c\x63ommit_errorR\x11pre_receive_errorR\x16\x63reate_tree_error_code\"\x8f\x02\n\x13UserCherryPickError\x12:\n\x14\x63herry_pick_conflict\x18\x01 \x01(\x0b\x32\x1a.gitaly.MergeConflictErrorH\x00\x12:\n\x16target_branch_diverged\x18\x02 \x01(\x0b\x32\x18.gitaly.NotAncestorErrorH\x00\x12\x45\n\x17\x63hanges_already_applied\x18\x03 \x01(\x0b\x32\".gitaly.ChangesAlreadyAppliedErrorH\x00\x12\x30\n\x0c\x61\x63\x63\x65ss_check\x18\x04 \x01(\x0b\x32\x18.gitaly.AccessCheckErrorH\x00\x42\x07\n\x05\x65rror\"\xc9\x02\n\x11UserRevertRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12!\n\x06\x63ommit\x18\x03 \x01(\x0b\x32\x11.gitaly.GitCommit\x12\x13\n\x0b\x62ranch_name\x18\x04 \x01(\x0c\x12\x0f\n\x07message\x18\x05 \x01(\x0c\x12\x19\n\x11start_branch_name\x18\x06 \x01(\x0c\x12,\n\x10start_repository\x18\x07 \x01(\x0b\x32\x12.gitaly.Repository\x12\x0f\n\x07\x64ry_run\x18\x08 \x01(\x08\x12-\n\ttimestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\n \x01(\t\"\x98\x02\n\x12UserRevertResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdate\x12\x19\n\x11\x63reate_tree_error\x18\x02 \x01(\t\x12\x14\n\x0c\x63ommit_error\x18\x03 \x01(\t\x12\x19\n\x11pre_receive_error\x18\x04 \x01(\t\x12J\n\x16\x63reate_tree_error_code\x18\x05 \x01(\x0e\x32*.gitaly.UserRevertResponse.CreateTreeError\"4\n\x0f\x43reateTreeError\x12\x08\n\x04NONE\x10\x00\x12\t\n\x05\x45MPTY\x10\x01\x12\x0c\n\x08\x43ONFLICT\x10\x02\"\xf9\x01\n\x0fUserRevertError\x12\x34\n\x0emerge_conflict\x18\x01 \x01(\x0b\x32\x1a.gitaly.MergeConflictErrorH\x00\x12\x45\n\x17\x63hanges_already_applied\x18\x02 \x01(\x0b\x32\".gitaly.ChangesAlreadyAppliedErrorH\x00\x12.\n\x0b\x63ustom_hook\x18\x03 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x12\x30\n\x0cnot_ancestor\x18\x04 \x01(\x0b\x32\x18.gitaly.NotAncestorErrorH\x00\x42\x07\n\x05\x65rror\"\xa7\x02\n\x1bUserCommitFilesActionHeader\x12>\n\x06\x61\x63tion\x18\x01 \x01(\x0e\x32..gitaly.UserCommitFilesActionHeader.ActionType\x12\x11\n\tfile_path\x18\x02 \x01(\x0c\x12\x15\n\rprevious_path\x18\x03 \x01(\x0c\x12\x16\n\x0e\x62\x61se64_content\x18\x04 \x01(\x08\x12\x18\n\x10\x65xecute_filemode\x18\x05 \x01(\x08\x12\x15\n\rinfer_content\x18\x06 \x01(\x08\"U\n\nActionType\x12\n\n\x06\x43REATE\x10\x00\x12\x0e\n\nCREATE_DIR\x10\x01\x12\n\n\x06UPDATE\x10\x02\x12\x08\n\x04MOVE\x10\x03\x12\n\n\x06\x44\x45LETE\x10\x04\x12\t\n\x05\x43HMOD\x10\x05\"\x85\x01\n\x15UserCommitFilesAction\x12\x35\n\x06header\x18\x01 \x01(\x0b\x32#.gitaly.UserCommitFilesActionHeaderH\x00\x12\x11\n\x07\x63ontent\x18\x02 \x01(\x0cH\x00\x42\"\n user_commit_files_action_payload\"\x90\x03\n\x1cUserCommitFilesRequestHeader\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x13\n\x0b\x62ranch_name\x18\x03 \x01(\x0c\x12\x16\n\x0e\x63ommit_message\x18\x04 \x01(\x0c\x12\x1a\n\x12\x63ommit_author_name\x18\x05 \x01(\x0c\x12\x1b\n\x13\x63ommit_author_email\x18\x06 \x01(\x0c\x12\x19\n\x11start_branch_name\x18\x07 \x01(\x0c\x12,\n\x10start_repository\x18\x08 \x01(\x0b\x32\x12.gitaly.Repository\x12\r\n\x05\x66orce\x18\t \x01(\x08\x12\x11\n\tstart_sha\x18\n \x01(\t\x12-\n\ttimestamp\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\x0c \x01(\t\x12\x0c\n\x04sign\x18\r \x01(\x08\"\xa6\x01\n\x16UserCommitFilesRequest\x12\x36\n\x06header\x18\x01 \x01(\x0b\x32$.gitaly.UserCommitFilesRequestHeaderH\x00\x12/\n\x06\x61\x63tion\x18\x02 \x01(\x0b\x32\x1d.gitaly.UserCommitFilesActionH\x00\x42#\n!user_commit_files_request_payload\"\x7f\n\x17UserCommitFilesResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdate\x12\x13\n\x0bindex_error\x18\x02 \x01(\t\x12\x19\n\x11pre_receive_error\x18\x03 \x01(\t\"\xad\x01\n\x14UserCommitFilesError\x12\x30\n\x0c\x61\x63\x63\x65ss_check\x18\x01 \x01(\x0b\x32\x18.gitaly.AccessCheckErrorH\x00\x12*\n\x0cindex_update\x18\x02 \x01(\x0b\x32\x12.gitaly.IndexErrorH\x00\x12.\n\x0b\x63ustom_hook\x18\x03 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x42\x07\n\x05\x65rror\"\xb8\x03\n\x1cUserRebaseConfirmableRequest\x12=\n\x06header\x18\x01 \x01(\x0b\x32+.gitaly.UserRebaseConfirmableRequest.HeaderH\x00\x12\x0f\n\x05\x61pply\x18\x02 \x01(\x08H\x00\x1a\x9c\x02\n\x06Header\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x15\n\trebase_id\x18\x03 \x01(\tB\x02\x18\x01\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\x0c\x12\x12\n\nbranch_sha\x18\x05 \x01(\t\x12-\n\x11remote_repository\x18\x06 \x01(\x0b\x32\x12.gitaly.Repository\x12\x15\n\rremote_branch\x18\x07 \x01(\x0c\x12\x18\n\x10git_push_options\x18\x08 \x03(\t\x12-\n\ttimestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampB)\n\'user_rebase_confirmable_request_payload\"\xa5\x01\n\x1dUserRebaseConfirmableResponse\x12\x14\n\nrebase_sha\x18\x01 \x01(\tH\x00\x12\x18\n\x0erebase_applied\x18\x02 \x01(\x08H\x00\x42*\n(user_rebase_confirmable_response_payloadJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05R\x11pre_receive_errorR\tgit_error\"\xfd\x01\n\x11UserSquashRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x11\n\tstart_sha\x18\x05 \x01(\t\x12\x0f\n\x07\x65nd_sha\x18\x06 \x01(\t\x12\x1c\n\x06\x61uthor\x18\x07 \x01(\x0b\x32\x0c.gitaly.User\x12\x16\n\x0e\x63ommit_message\x18\x08 \x01(\x0c\x12-\n\ttimestamp\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampJ\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05R\tsquash_id\"R\n\x12UserSquashResponse\x12\x12\n\nsquash_sha\x18\x01 \x01(\tJ\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04R\x11pre_receive_errorR\tgit_error\"\x8e\x01\n\x1aUserRebaseConfirmableError\x12\x35\n\x0frebase_conflict\x18\x01 \x01(\x0b\x32\x1a.gitaly.MergeConflictErrorH\x00\x12\x30\n\x0c\x61\x63\x63\x65ss_check\x18\x02 \x01(\x0b\x32\x18.gitaly.AccessCheckErrorH\x00\x42\x07\n\x05\x65rror\"\x8b\x01\n\x0fUserSquashError\x12\x38\n\x10resolve_revision\x18\x01 \x01(\x0b\x32\x1c.gitaly.ResolveRevisionErrorH\x00\x12\x35\n\x0frebase_conflict\x18\x02 \x01(\x0b\x32\x1a.gitaly.MergeConflictErrorH\x00\x42\x07\n\x05\x65rror\"\xbb\x02\n\x15UserApplyPatchRequest\x12\x36\n\x06header\x18\x01 \x01(\x0b\x32$.gitaly.UserApplyPatchRequest.HeaderH\x00\x12\x11\n\x07patches\x18\x02 \x01(\x0cH\x00\x1a\xb2\x01\n\x06Header\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x15\n\rtarget_branch\x18\x03 \x01(\x0c\x12-\n\ttimestamp\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\x05 \x01(\tB\"\n user_apply_patch_request_payload\"N\n\x16UserApplyPatchResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdate\"\xfe\x01\n\x1aUserUpdateSubmoduleRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x12\n\ncommit_sha\x18\x03 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x04 \x01(\x0c\x12\x11\n\tsubmodule\x18\x05 \x01(\x0c\x12\x16\n\x0e\x63ommit_message\x18\x06 \x01(\x0c\x12-\n\ttimestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\x08 \x01(\t\"\x9d\x01\n\x1bUserUpdateSubmoduleResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdate\x12\x19\n\x11pre_receive_error\x18\x02 \x01(\t\x12\x14\n\x0c\x63ommit_error\x18\x04 \x01(\tJ\x04\x08\x03\x10\x04R\x11\x63reate_tree_error2\xc9\x0b\n\x10OperationService\x12]\n\x10UserCreateBranch\x12\x1f.gitaly.UserCreateBranchRequest\x1a .gitaly.UserCreateBranchResponse\"\x06\xfa\x97(\x02\x08\x01\x12]\n\x10UserUpdateBranch\x12\x1f.gitaly.UserUpdateBranchRequest\x1a .gitaly.UserUpdateBranchResponse\"\x06\xfa\x97(\x02\x08\x01\x12]\n\x10UserDeleteBranch\x12\x1f.gitaly.UserDeleteBranchRequest\x1a .gitaly.UserDeleteBranchResponse\"\x06\xfa\x97(\x02\x08\x01\x12T\n\rUserCreateTag\x12\x1c.gitaly.UserCreateTagRequest\x1a\x1d.gitaly.UserCreateTagResponse\"\x06\xfa\x97(\x02\x08\x01\x12T\n\rUserDeleteTag\x12\x1c.gitaly.UserDeleteTagRequest\x1a\x1d.gitaly.UserDeleteTagResponse\"\x06\xfa\x97(\x02\x08\x01\x12W\n\x0eUserMergeToRef\x12\x1d.gitaly.UserMergeToRefRequest\x1a\x1e.gitaly.UserMergeToRefResponse\"\x06\xfa\x97(\x02\x08\x01\x12Z\n\x0fUserRebaseToRef\x12\x1e.gitaly.UserRebaseToRefRequest\x1a\x1f.gitaly.UserRebaseToRefResponse\"\x06\xfa\x97(\x02\x08\x01\x12^\n\x0fUserMergeBranch\x12\x1e.gitaly.UserMergeBranchRequest\x1a\x1f.gitaly.UserMergeBranchResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x30\x01\x12Q\n\x0cUserFFBranch\x12\x1b.gitaly.UserFFBranchRequest\x1a\x1c.gitaly.UserFFBranchResponse\"\x06\xfa\x97(\x02\x08\x01\x12W\n\x0eUserCherryPick\x12\x1d.gitaly.UserCherryPickRequest\x1a\x1e.gitaly.UserCherryPickResponse\"\x06\xfa\x97(\x02\x08\x01\x12\\\n\x0fUserCommitFiles\x12\x1e.gitaly.UserCommitFilesRequest\x1a\x1f.gitaly.UserCommitFilesResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x12p\n\x15UserRebaseConfirmable\x12$.gitaly.UserRebaseConfirmableRequest\x1a%.gitaly.UserRebaseConfirmableResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x30\x01\x12K\n\nUserRevert\x12\x19.gitaly.UserRevertRequest\x1a\x1a.gitaly.UserRevertResponse\"\x06\xfa\x97(\x02\x08\x01\x12K\n\nUserSquash\x12\x19.gitaly.UserSquashRequest\x1a\x1a.gitaly.UserSquashResponse\"\x06\xfa\x97(\x02\x08\x01\x12Y\n\x0eUserApplyPatch\x12\x1d.gitaly.UserApplyPatchRequest\x1a\x1e.gitaly.UserApplyPatchResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x12\x66\n\x13UserUpdateSubmodule\x12\".gitaly.UserUpdateSubmoduleRequest\x1a#.gitaly.UserUpdateSubmoduleResponse\"\x06\xfa\x97(\x02\x08\x01\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'operations_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _USERCREATEBRANCHREQUEST.fields_by_name['repository']._options = None
  _USERCREATEBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERUPDATEBRANCHREQUEST.fields_by_name['repository']._options = None
  _USERUPDATEBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERDELETEBRANCHREQUEST.fields_by_name['repository']._options = None
  _USERDELETEBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERDELETETAGREQUEST.fields_by_name['repository']._options = None
  _USERDELETETAGREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERCREATETAGREQUEST.fields_by_name['repository']._options = None
  _USERCREATETAGREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERMERGEBRANCHREQUEST.fields_by_name['repository']._options = None
  _USERMERGEBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERMERGETOREFREQUEST.fields_by_name['repository']._options = None
  _USERMERGETOREFREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERMERGETOREFREQUEST.fields_by_name['branch']._options = None
  _USERMERGETOREFREQUEST.fields_by_name['branch']._serialized_options = b'\030\001'
  _USERMERGETOREFREQUEST.fields_by_name['allow_conflicts']._options = None
  _USERMERGETOREFREQUEST.fields_by_name['allow_conflicts']._serialized_options = b'\030\001'
  _USERREBASETOREFREQUEST.fields_by_name['repository']._options = None
  _USERREBASETOREFREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERFFBRANCHREQUEST.fields_by_name['repository']._options = None
  _USERFFBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERCHERRYPICKREQUEST.fields_by_name['repository']._options = None
  _USERCHERRYPICKREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERREVERTREQUEST.fields_by_name['repository']._options = None
  _USERREVERTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERCOMMITFILESREQUESTHEADER.fields_by_name['repository']._options = None
  _USERCOMMITFILESREQUESTHEADER.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERREBASECONFIRMABLEREQUEST_HEADER.fields_by_name['repository']._options = None
  _USERREBASECONFIRMABLEREQUEST_HEADER.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERREBASECONFIRMABLEREQUEST_HEADER.fields_by_name['rebase_id']._options = None
  _USERREBASECONFIRMABLEREQUEST_HEADER.fields_by_name['rebase_id']._serialized_options = b'\030\001'
  _USERSQUASHREQUEST.fields_by_name['repository']._options = None
  _USERSQUASHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERAPPLYPATCHREQUEST_HEADER.fields_by_name['repository']._options = None
  _USERAPPLYPATCHREQUEST_HEADER.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _USERUPDATESUBMODULEREQUEST.fields_by_name['repository']._options = None
  _USERUPDATESUBMODULEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _OPERATIONSERVICE.methods_by_name['UserCreateBranch']._options = None
  _OPERATIONSERVICE.methods_by_name['UserCreateBranch']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserUpdateBranch']._options = None
  _OPERATIONSERVICE.methods_by_name['UserUpdateBranch']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserDeleteBranch']._options = None
  _OPERATIONSERVICE.methods_by_name['UserDeleteBranch']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserCreateTag']._options = None
  _OPERATIONSERVICE.methods_by_name['UserCreateTag']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserDeleteTag']._options = None
  _OPERATIONSERVICE.methods_by_name['UserDeleteTag']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserMergeToRef']._options = None
  _OPERATIONSERVICE.methods_by_name['UserMergeToRef']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserRebaseToRef']._options = None
  _OPERATIONSERVICE.methods_by_name['UserRebaseToRef']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserMergeBranch']._options = None
  _OPERATIONSERVICE.methods_by_name['UserMergeBranch']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserFFBranch']._options = None
  _OPERATIONSERVICE.methods_by_name['UserFFBranch']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserCherryPick']._options = None
  _OPERATIONSERVICE.methods_by_name['UserCherryPick']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserCommitFiles']._options = None
  _OPERATIONSERVICE.methods_by_name['UserCommitFiles']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserRebaseConfirmable']._options = None
  _OPERATIONSERVICE.methods_by_name['UserRebaseConfirmable']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserRevert']._options = None
  _OPERATIONSERVICE.methods_by_name['UserRevert']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserSquash']._options = None
  _OPERATIONSERVICE.methods_by_name['UserSquash']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserApplyPatch']._options = None
  _OPERATIONSERVICE.methods_by_name['UserApplyPatch']._serialized_options = b'\372\227(\002\010\001'
  _OPERATIONSERVICE.methods_by_name['UserUpdateSubmodule']._options = None
  _OPERATIONSERVICE.methods_by_name['UserUpdateSubmodule']._serialized_options = b'\372\227(\002\010\001'
  _globals['_USERCREATEBRANCHREQUEST']._serialized_start=102
  _globals['_USERCREATEBRANCHREQUEST']._serialized_end=243
  _globals['_USERCREATEBRANCHRESPONSE']._serialized_start=245
  _globals['_USERCREATEBRANCHRESPONSE']._serialized_end=328
  _globals['_USERCREATEBRANCHERROR']._serialized_start=330
  _globals['_USERCREATEBRANCHERROR']._serialized_end=410
  _globals['_USERUPDATEBRANCHREQUEST']._serialized_start=413
  _globals['_USERUPDATEBRANCHREQUEST']._serialized_end=565
  _globals['_USERUPDATEBRANCHRESPONSE']._serialized_start=567
  _globals['_USERUPDATEBRANCHRESPONSE']._serialized_end=620
  _globals['_USERDELETEBRANCHREQUEST']._serialized_start=623
  _globals['_USERDELETEBRANCHREQUEST']._serialized_end=769
  _globals['_USERDELETEBRANCHRESPONSE']._serialized_start=771
  _globals['_USERDELETEBRANCHRESPONSE']._serialized_end=822
  _globals['_USERDELETEBRANCHERROR']._serialized_start=825
  _globals['_USERDELETEBRANCHERROR']._serialized_end=1013
  _globals['_USERDELETETAGREQUEST']._serialized_start=1016
  _globals['_USERDELETETAGREQUEST']._serialized_end=1156
  _globals['_USERDELETETAGRESPONSE']._serialized_start=1158
  _globals['_USERDELETETAGRESPONSE']._serialized_end=1208
  _globals['_USERCREATETAGREQUEST']._serialized_start=1211
  _globals['_USERCREATETAGREQUEST']._serialized_end=1414
  _globals['_USERCREATETAGRESPONSE']._serialized_start=1416
  _globals['_USERCREATETAGRESPONSE']._serialized_end=1504
  _globals['_USERCREATETAGERROR']._serialized_start=1507
  _globals['_USERCREATETAGERROR']._serialized_end=1750
  _globals['_USERMERGEBRANCHREQUEST']._serialized_start=1753
  _globals['_USERMERGEBRANCHREQUEST']._serialized_end=1991
  _globals['_USERMERGEBRANCHRESPONSE']._serialized_start=1994
  _globals['_USERMERGEBRANCHRESPONSE']._serialized_end=2123
  _globals['_USERMERGEBRANCHERROR']._serialized_start=2126
  _globals['_USERMERGEBRANCHERROR']._serialized_end=2367
  _globals['_USERMERGETOREFREQUEST']._serialized_start=2370
  _globals['_USERMERGETOREFREQUEST']._serialized_end=2672
  _globals['_USERMERGETOREFRESPONSE']._serialized_start=2674
  _globals['_USERMERGETOREFRESPONSE']._serialized_end=2742
  _globals['_USERREBASETOREFREQUEST']._serialized_start=2745
  _globals['_USERREBASETOREFREQUEST']._serialized_end=2982
  _globals['_USERREBASETOREFRESPONSE']._serialized_start=2984
  _globals['_USERREBASETOREFRESPONSE']._serialized_end=3028
  _globals['_OPERATIONBRANCHUPDATE']._serialized_start=3030
  _globals['_OPERATIONBRANCHUPDATE']._serialized_end=3118
  _globals['_USERFFBRANCHREQUEST']._serialized_start=3121
  _globals['_USERFFBRANCHREQUEST']._serialized_end=3277
  _globals['_USERFFBRANCHRESPONSE']._serialized_start=3279
  _globals['_USERFFBRANCHRESPONSE']._serialized_end=3382
  _globals['_USERCHERRYPICKREQUEST']._serialized_start=3385
  _globals['_USERCHERRYPICKREQUEST']._serialized_end=3775
  _globals['_USERCHERRYPICKRESPONSE']._serialized_start=3778
  _globals['_USERCHERRYPICKRESPONSE']._serialized_end=3956
  _globals['_USERCHERRYPICKERROR']._serialized_start=3959
  _globals['_USERCHERRYPICKERROR']._serialized_end=4230
  _globals['_USERREVERTREQUEST']._serialized_start=4233
  _globals['_USERREVERTREQUEST']._serialized_end=4562
  _globals['_USERREVERTRESPONSE']._serialized_start=4565
  _globals['_USERREVERTRESPONSE']._serialized_end=4845
  _globals['_USERREVERTRESPONSE_CREATETREEERROR']._serialized_start=4793
  _globals['_USERREVERTRESPONSE_CREATETREEERROR']._serialized_end=4845
  _globals['_USERREVERTERROR']._serialized_start=4848
  _globals['_USERREVERTERROR']._serialized_end=5097
  _globals['_USERCOMMITFILESACTIONHEADER']._serialized_start=5100
  _globals['_USERCOMMITFILESACTIONHEADER']._serialized_end=5395
  _globals['_USERCOMMITFILESACTIONHEADER_ACTIONTYPE']._serialized_start=5310
  _globals['_USERCOMMITFILESACTIONHEADER_ACTIONTYPE']._serialized_end=5395
  _globals['_USERCOMMITFILESACTION']._serialized_start=5398
  _globals['_USERCOMMITFILESACTION']._serialized_end=5531
  _globals['_USERCOMMITFILESREQUESTHEADER']._serialized_start=5534
  _globals['_USERCOMMITFILESREQUESTHEADER']._serialized_end=5934
  _globals['_USERCOMMITFILESREQUEST']._serialized_start=5937
  _globals['_USERCOMMITFILESREQUEST']._serialized_end=6103
  _globals['_USERCOMMITFILESRESPONSE']._serialized_start=6105
  _globals['_USERCOMMITFILESRESPONSE']._serialized_end=6232
  _globals['_USERCOMMITFILESERROR']._serialized_start=6235
  _globals['_USERCOMMITFILESERROR']._serialized_end=6408
  _globals['_USERREBASECONFIRMABLEREQUEST']._serialized_start=6411
  _globals['_USERREBASECONFIRMABLEREQUEST']._serialized_end=6851
  _globals['_USERREBASECONFIRMABLEREQUEST_HEADER']._serialized_start=6524
  _globals['_USERREBASECONFIRMABLEREQUEST_HEADER']._serialized_end=6808
  _globals['_USERREBASECONFIRMABLERESPONSE']._serialized_start=6854
  _globals['_USERREBASECONFIRMABLERESPONSE']._serialized_end=7019
  _globals['_USERSQUASHREQUEST']._serialized_start=7022
  _globals['_USERSQUASHREQUEST']._serialized_end=7275
  _globals['_USERSQUASHRESPONSE']._serialized_start=7277
  _globals['_USERSQUASHRESPONSE']._serialized_end=7359
  _globals['_USERREBASECONFIRMABLEERROR']._serialized_start=7362
  _globals['_USERREBASECONFIRMABLEERROR']._serialized_end=7504
  _globals['_USERSQUASHERROR']._serialized_start=7507
  _globals['_USERSQUASHERROR']._serialized_end=7646
  _globals['_USERAPPLYPATCHREQUEST']._serialized_start=7649
  _globals['_USERAPPLYPATCHREQUEST']._serialized_end=7964
  _globals['_USERAPPLYPATCHREQUEST_HEADER']._serialized_start=7750
  _globals['_USERAPPLYPATCHREQUEST_HEADER']._serialized_end=7928
  _globals['_USERAPPLYPATCHRESPONSE']._serialized_start=7966
  _globals['_USERAPPLYPATCHRESPONSE']._serialized_end=8044
  _globals['_USERUPDATESUBMODULEREQUEST']._serialized_start=8047
  _globals['_USERUPDATESUBMODULEREQUEST']._serialized_end=8301
  _globals['_USERUPDATESUBMODULERESPONSE']._serialized_start=8304
  _globals['_USERUPDATESUBMODULERESPONSE']._serialized_end=8461
  _globals['_OPERATIONSERVICE']._serialized_start=8464
  _globals['_OPERATIONSERVICE']._serialized_end=9945
# @@protoc_insertion_point(module_scope)
