# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mercurial-changeset.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19mercurial-changeset.proto\x12\x07hgitaly\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\nlint.proto\x1a\x0cshared.proto\"\x9b\x02\n\x12MercurialChangeset\x12\n\n\x02id\x18\x01 \x01(\t\x12\x12\n\nparent_ids\x18\x02 \x03(\t\x12\r\n\x05title\x18\x03 \x01(\x0c\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\x0c\x12\x0e\n\x06\x61uthor\x18\x05 \x01(\x0c\x12(\n\x04\x64\x61te\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06\x62ranch\x18\x07 \x01(\x0c\x12\r\n\x05topic\x18\x08 \x01(\x0c\x12\x10\n\x08obsolete\x18\t \x01(\x08\x12\x0c\n\x04\x66qbn\x18\n \x01(\x0c\x12\x17\n\x0ftopic_namespace\x18\x0b \x01(\x0c\x12/\n\x05phase\x18\x0c \x01(\x0e\x32 .hgitaly.MercurialChangesetPhase\"\xc0\x01\n\x1eListMercurialChangesetsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12.\n\x04view\x18\x02 \x01(\x0e\x32 .hgitaly.MercurialRepositoryView\x12\x0e\n\x06revset\x18\x03 \x01(\x0c\x12\x30\n\x06\x66ields\x18\x04 \x03(\x0e\x32 .hgitaly.MercurialChangesetField\"R\n\x1fListMercurialChangesetsResponse\x12/\n\nchangesets\x18\x01 \x03(\x0b\x32\x1b.hgitaly.MercurialChangeset*X\n\x17MercurialChangesetPhase\x12\n\n\x06PUBLIC\x10\x00\x12\t\n\x05\x44RAFT\x10\x01\x12\n\n\x06SECRET\x10\x02\x12\x0c\n\x08\x41RCHIVED\x10 \x12\x0c\n\x08INTERNAL\x10`*\xb3\x01\n\x17MercurialChangesetField\x12\x07\n\x03\x41LL\x10\x00\x12\x0e\n\nPARENT_IDS\x10\x01\x12\t\n\x05TITLE\x10\x02\x12\x0f\n\x0b\x44\x45SCRIPTION\x10\x03\x12\n\n\x06\x41UTHOR\x10\x04\x12\x08\n\x04\x44\x41TE\x10\x05\x12\n\n\x06\x42RANCH\x10\x06\x12\t\n\x05TOPIC\x10\x07\x12\x0c\n\x08OBSOLETE\x10\x08\x12\x08\n\x04\x46QBN\x10\t\x12\x13\n\x0fTOPIC_NAMESPACE\x10\n\x12\t\n\x05PHASE\x10\x0b*6\n\x17MercurialRepositoryView\x12\x0b\n\x07VISIBLE\x10\x00\x12\x0e\n\nUNFILTERED\x10\x01\x32\x93\x01\n\x19MercurialChangesetService\x12v\n\x17ListMercurialChangesets\x12\'.hgitaly.ListMercurialChangesetsRequest\x1a(.hgitaly.ListMercurialChangesetsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mercurial_changeset_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _LISTMERCURIALCHANGESETSREQUEST.fields_by_name['repository']._options = None
  _LISTMERCURIALCHANGESETSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _MERCURIALCHANGESETSERVICE.methods_by_name['ListMercurialChangesets']._options = None
  _MERCURIALCHANGESETSERVICE.methods_by_name['ListMercurialChangesets']._serialized_options = b'\372\227(\002\010\002'
  _globals['_MERCURIALCHANGESETPHASE']._serialized_start=662
  _globals['_MERCURIALCHANGESETPHASE']._serialized_end=750
  _globals['_MERCURIALCHANGESETFIELD']._serialized_start=753
  _globals['_MERCURIALCHANGESETFIELD']._serialized_end=932
  _globals['_MERCURIALREPOSITORYVIEW']._serialized_start=934
  _globals['_MERCURIALREPOSITORYVIEW']._serialized_end=988
  _globals['_MERCURIALCHANGESET']._serialized_start=98
  _globals['_MERCURIALCHANGESET']._serialized_end=381
  _globals['_LISTMERCURIALCHANGESETSREQUEST']._serialized_start=384
  _globals['_LISTMERCURIALCHANGESETSREQUEST']._serialized_end=576
  _globals['_LISTMERCURIALCHANGESETSRESPONSE']._serialized_start=578
  _globals['_LISTMERCURIALCHANGESETSRESPONSE']._serialized_end=660
  _globals['_MERCURIALCHANGESETSERVICE']._serialized_start=991
  _globals['_MERCURIALCHANGESETSERVICE']._serialized_end=1138
# @@protoc_insertion_point(module_scope)
