# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ref.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import errors_pb2 as errors__pb2
from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\tref.proto\x12\x06gitaly\x1a\x0c\x65rrors.proto\x1a\nlint.proto\x1a\x0cshared.proto\"_\n\x1c\x46indDefaultBranchNameRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\thead_only\x18\x02 \x01(\x08\"-\n\x1d\x46indDefaultBranchNameResponse\x12\x0c\n\x04name\x18\x01 \x01(\x0c\"\xf1\x01\n\x18\x46indLocalBranchesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x38\n\x07sort_by\x18\x02 \x01(\x0e\x32\'.gitaly.FindLocalBranchesRequest.SortBy\x12\x36\n\x11pagination_params\x18\x03 \x01(\x0b\x32\x1b.gitaly.PaginationParameter\"5\n\x06SortBy\x12\x08\n\x04NAME\x10\x00\x12\x0f\n\x0bUPDATED_ASC\x10\x01\x12\x10\n\x0cUPDATED_DESC\x10\x02\"S\n\x19\x46indLocalBranchesResponse\x12&\n\x0elocal_branches\x18\x02 \x03(\x0b\x32\x0e.gitaly.BranchJ\x04\x08\x01\x10\x02R\x08\x62ranches\"t\n\x16\x46indAllBranchesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x13\n\x0bmerged_only\x18\x02 \x01(\x08\x12\x17\n\x0fmerged_branches\x18\x03 \x03(\x0c\"\x8e\x01\n\x17\x46indAllBranchesResponse\x12\x38\n\x08\x62ranches\x18\x01 \x03(\x0b\x32&.gitaly.FindAllBranchesResponse.Branch\x1a\x39\n\x06\x42ranch\x12\x0c\n\x04name\x18\x01 \x01(\x0c\x12!\n\x06target\x18\x02 \x01(\x0b\x32\x11.gitaly.GitCommit\"P\n\x0e\x46indTagRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08tag_name\x18\x02 \x01(\x0c\"+\n\x0f\x46indTagResponse\x12\x18\n\x03tag\x18\x01 \x01(\x0b\x32\x0b.gitaly.Tag\"P\n\x0c\x46indTagError\x12\x37\n\rtag_not_found\x18\x01 \x01(\x0b\x32\x1e.gitaly.ReferenceNotFoundErrorH\x00\x42\x07\n\x05\x65rror\"\xd1\x02\n\x12\x46indAllTagsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x32\n\x07sort_by\x18\x02 \x01(\x0b\x32!.gitaly.FindAllTagsRequest.SortBy\x12\x36\n\x11pagination_params\x18\x03 \x01(\x0b\x32\x1b.gitaly.PaginationParameter\x1a\xa0\x01\n\x06SortBy\x12\x32\n\x03key\x18\x01 \x01(\x0e\x32%.gitaly.FindAllTagsRequest.SortBy.Key\x12(\n\tdirection\x18\x02 \x01(\x0e\x32\x15.gitaly.SortDirection\"8\n\x03Key\x12\x0b\n\x07REFNAME\x10\x00\x12\x0f\n\x0b\x43REATORDATE\x10\x01\x12\x13\n\x0fVERSION_REFNAME\x10\x02\"0\n\x13\x46indAllTagsResponse\x12\x19\n\x04tags\x18\x01 \x03(\x0b\x32\x0b.gitaly.Tag\"M\n\x10RefExistsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0b\n\x03ref\x18\x02 \x01(\x0c\"\"\n\x11RefExistsResponse\x12\r\n\x05value\x18\x01 \x01(\x08\"O\n\x11\x46indBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0c\n\x04name\x18\x02 \x01(\x0c\"4\n\x12\x46indBranchResponse\x12\x1e\n\x06\x62ranch\x18\x01 \x01(\x0b\x32\x0e.gitaly.Branch\"\xcb\x01\n\x17UpdateReferencesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x37\n\x07updates\x18\x02 \x03(\x0b\x32&.gitaly.UpdateReferencesRequest.Update\x1aI\n\x06Update\x12\x11\n\treference\x18\x01 \x01(\x0c\x12\x15\n\rold_object_id\x18\x02 \x01(\x0c\x12\x15\n\rnew_object_id\x18\x03 \x01(\x0c\"\x1a\n\x18UpdateReferencesResponse\"\xde\x01\n\x15UpdateReferencesError\x12\x37\n\x0einvalid_format\x18\x01 \x01(\x0b\x32\x1d.gitaly.InvalidRefFormatErrorH\x00\x12:\n\x11references_locked\x18\x02 \x01(\x0b\x32\x1d.gitaly.ReferencesLockedErrorH\x00\x12G\n\x18reference_state_mismatch\x18\x03 \x01(\x0b\x32#.gitaly.ReferenceStateMismatchErrorH\x00\x42\x07\n\x05\x65rror\"k\n\x11\x44\x65leteRefsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x12\x65xcept_with_prefix\x18\x02 \x03(\x0c\x12\x0c\n\x04refs\x18\x03 \x03(\x0c\"\'\n\x12\x44\x65leteRefsResponse\x12\x11\n\tgit_error\x18\x01 \x01(\t\"\x8f\x01\n\x0f\x44\x65leteRefsError\x12\x37\n\x0einvalid_format\x18\x01 \x01(\x0b\x32\x1d.gitaly.InvalidRefFormatErrorH\x00\x12:\n\x11references_locked\x18\x02 \x01(\x0b\x32\x1d.gitaly.ReferencesLockedErrorH\x00\x42\x07\n\x05\x65rror\"x\n&ListBranchNamesContainingCommitRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\tcommit_id\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\r\"E\n\'ListBranchNamesContainingCommitResponse\x12\x14\n\x0c\x62ranch_names\x18\x02 \x03(\x0cJ\x04\x08\x01\x10\x02\"u\n#ListTagNamesContainingCommitRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\tcommit_id\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\r\"?\n$ListTagNamesContainingCommitResponse\x12\x11\n\ttag_names\x18\x02 \x03(\x0cJ\x04\x08\x01\x10\x02\"^\n\x17GetTagSignaturesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x15\n\rtag_revisions\x18\x02 \x03(\t\"\xa1\x01\n\x18GetTagSignaturesResponse\x12\x41\n\nsignatures\x18\x01 \x03(\x0b\x32-.gitaly.GetTagSignaturesResponse.TagSignature\x1a\x42\n\x0cTagSignature\x12\x0e\n\x06tag_id\x18\x01 \x01(\t\x12\x11\n\tsignature\x18\x02 \x01(\x0c\x12\x0f\n\x07\x63ontent\x18\x03 \x01(\x0c\"g\n\x15GetTagMessagesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0f\n\x07tag_ids\x18\x03 \x03(\tJ\x04\x08\x02\x10\x03R\ttag_names\"I\n\x16GetTagMessagesResponse\x12\x0f\n\x07message\x18\x02 \x01(\x0c\x12\x0e\n\x06tag_id\x18\x03 \x01(\tJ\x04\x08\x01\x10\x02R\x08tag_name\"a\n\x1c\x46indAllRemoteBranchesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x13\n\x0bremote_name\x18\x02 \x01(\t\"A\n\x1d\x46indAllRemoteBranchesResponse\x12 \n\x08\x62ranches\x18\x01 \x03(\x0b\x32\x0e.gitaly.Branch\"\xeb\x02\n\x0fListRefsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08patterns\x18\x02 \x03(\x0c\x12\x0c\n\x04head\x18\x03 \x01(\x08\x12/\n\x07sort_by\x18\x04 \x01(\x0b\x32\x1e.gitaly.ListRefsRequest.SortBy\x12\x18\n\x10pointing_at_oids\x18\x05 \x03(\x0c\x12\x11\n\tpeel_tags\x18\x06 \x01(\x08\x1a\xab\x01\n\x06SortBy\x12/\n\x03key\x18\x01 \x01(\x0e\x32\".gitaly.ListRefsRequest.SortBy.Key\x12(\n\tdirection\x18\x02 \x01(\x0e\x32\x15.gitaly.SortDirection\"F\n\x03Key\x12\x0b\n\x07REFNAME\x10\x00\x12\x0f\n\x0b\x43REATORDATE\x10\x01\x12\x0e\n\nAUTHORDATE\x10\x02\x12\x11\n\rCOMMITTERDATE\x10\x03\"\x8c\x01\n\x10ListRefsResponse\x12\x36\n\nreferences\x18\x01 \x03(\x0b\x32\".gitaly.ListRefsResponse.Reference\x1a@\n\tReference\x12\x0c\n\x04name\x18\x01 \x01(\x0c\x12\x0e\n\x06target\x18\x02 \x01(\t\x12\x15\n\rpeeled_target\x18\x03 \x01(\t\"\x8a\x01\n\x14\x46indRefsByOIDRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0b\n\x03oid\x18\x02 \x01(\t\x12\x14\n\x0cref_patterns\x18\x03 \x03(\t\x12\x12\n\nsort_field\x18\x04 \x01(\t\x12\r\n\x05limit\x18\x05 \x01(\r\"%\n\x15\x46indRefsByOIDResponse\x12\x0c\n\x04refs\x18\x01 \x03(\t2\xf7\x0b\n\nRefService\x12l\n\x15\x46indDefaultBranchName\x12$.gitaly.FindDefaultBranchNameRequest\x1a%.gitaly.FindDefaultBranchNameResponse\"\x06\xfa\x97(\x02\x08\x02\x12\x62\n\x11\x46indLocalBranches\x12 .gitaly.FindLocalBranchesRequest\x1a!.gitaly.FindLocalBranchesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\\\n\x0f\x46indAllBranches\x12\x1e.gitaly.FindAllBranchesRequest\x1a\x1f.gitaly.FindAllBranchesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12P\n\x0b\x46indAllTags\x12\x1a.gitaly.FindAllTagsRequest\x1a\x1b.gitaly.FindAllTagsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\x42\n\x07\x46indTag\x12\x16.gitaly.FindTagRequest\x1a\x17.gitaly.FindTagResponse\"\x06\xfa\x97(\x02\x08\x02\x12n\n\x15\x46indAllRemoteBranches\x12$.gitaly.FindAllRemoteBranchesRequest\x1a%.gitaly.FindAllRemoteBranchesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12H\n\tRefExists\x12\x18.gitaly.RefExistsRequest\x1a\x19.gitaly.RefExistsResponse\"\x06\xfa\x97(\x02\x08\x02\x12K\n\nFindBranch\x12\x19.gitaly.FindBranchRequest\x1a\x1a.gitaly.FindBranchResponse\"\x06\xfa\x97(\x02\x08\x02\x12_\n\x10UpdateReferences\x12\x1f.gitaly.UpdateReferencesRequest\x1a .gitaly.UpdateReferencesResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x12K\n\nDeleteRefs\x12\x19.gitaly.DeleteRefsRequest\x1a\x1a.gitaly.DeleteRefsResponse\"\x06\xfa\x97(\x02\x08\x01\x12\x8c\x01\n\x1fListBranchNamesContainingCommit\x12..gitaly.ListBranchNamesContainingCommitRequest\x1a/.gitaly.ListBranchNamesContainingCommitResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\x83\x01\n\x1cListTagNamesContainingCommit\x12+.gitaly.ListTagNamesContainingCommitRequest\x1a,.gitaly.ListTagNamesContainingCommitResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12_\n\x10GetTagSignatures\x12\x1f.gitaly.GetTagSignaturesRequest\x1a .gitaly.GetTagSignaturesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12Y\n\x0eGetTagMessages\x12\x1d.gitaly.GetTagMessagesRequest\x1a\x1e.gitaly.GetTagMessagesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12G\n\x08ListRefs\x12\x17.gitaly.ListRefsRequest\x1a\x18.gitaly.ListRefsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12T\n\rFindRefsByOID\x12\x1c.gitaly.FindRefsByOIDRequest\x1a\x1d.gitaly.FindRefsByOIDResponse\"\x06\xfa\x97(\x02\x08\x02\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ref_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _FINDDEFAULTBRANCHNAMEREQUEST.fields_by_name['repository']._options = None
  _FINDDEFAULTBRANCHNAMEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDLOCALBRANCHESREQUEST.fields_by_name['repository']._options = None
  _FINDLOCALBRANCHESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDALLBRANCHESREQUEST.fields_by_name['repository']._options = None
  _FINDALLBRANCHESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDTAGREQUEST.fields_by_name['repository']._options = None
  _FINDTAGREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDALLTAGSREQUEST.fields_by_name['repository']._options = None
  _FINDALLTAGSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REFEXISTSREQUEST.fields_by_name['repository']._options = None
  _REFEXISTSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDBRANCHREQUEST.fields_by_name['repository']._options = None
  _FINDBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _UPDATEREFERENCESREQUEST.fields_by_name['repository']._options = None
  _UPDATEREFERENCESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _DELETEREFSREQUEST.fields_by_name['repository']._options = None
  _DELETEREFSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTBRANCHNAMESCONTAININGCOMMITREQUEST.fields_by_name['repository']._options = None
  _LISTBRANCHNAMESCONTAININGCOMMITREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTTAGNAMESCONTAININGCOMMITREQUEST.fields_by_name['repository']._options = None
  _LISTTAGNAMESCONTAININGCOMMITREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETTAGSIGNATURESREQUEST.fields_by_name['repository']._options = None
  _GETTAGSIGNATURESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETTAGMESSAGESREQUEST.fields_by_name['repository']._options = None
  _GETTAGMESSAGESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDALLREMOTEBRANCHESREQUEST.fields_by_name['repository']._options = None
  _FINDALLREMOTEBRANCHESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTREFSREQUEST.fields_by_name['repository']._options = None
  _LISTREFSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDREFSBYOIDREQUEST.fields_by_name['repository']._options = None
  _FINDREFSBYOIDREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REFSERVICE.methods_by_name['FindDefaultBranchName']._options = None
  _REFSERVICE.methods_by_name['FindDefaultBranchName']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindLocalBranches']._options = None
  _REFSERVICE.methods_by_name['FindLocalBranches']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindAllBranches']._options = None
  _REFSERVICE.methods_by_name['FindAllBranches']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindAllTags']._options = None
  _REFSERVICE.methods_by_name['FindAllTags']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindTag']._options = None
  _REFSERVICE.methods_by_name['FindTag']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindAllRemoteBranches']._options = None
  _REFSERVICE.methods_by_name['FindAllRemoteBranches']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['RefExists']._options = None
  _REFSERVICE.methods_by_name['RefExists']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindBranch']._options = None
  _REFSERVICE.methods_by_name['FindBranch']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['UpdateReferences']._options = None
  _REFSERVICE.methods_by_name['UpdateReferences']._serialized_options = b'\372\227(\002\010\001'
  _REFSERVICE.methods_by_name['DeleteRefs']._options = None
  _REFSERVICE.methods_by_name['DeleteRefs']._serialized_options = b'\372\227(\002\010\001'
  _REFSERVICE.methods_by_name['ListBranchNamesContainingCommit']._options = None
  _REFSERVICE.methods_by_name['ListBranchNamesContainingCommit']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['ListTagNamesContainingCommit']._options = None
  _REFSERVICE.methods_by_name['ListTagNamesContainingCommit']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['GetTagSignatures']._options = None
  _REFSERVICE.methods_by_name['GetTagSignatures']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['GetTagMessages']._options = None
  _REFSERVICE.methods_by_name['GetTagMessages']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['ListRefs']._options = None
  _REFSERVICE.methods_by_name['ListRefs']._serialized_options = b'\372\227(\002\010\002'
  _REFSERVICE.methods_by_name['FindRefsByOID']._options = None
  _REFSERVICE.methods_by_name['FindRefsByOID']._serialized_options = b'\372\227(\002\010\002'
  _globals['_FINDDEFAULTBRANCHNAMEREQUEST']._serialized_start=61
  _globals['_FINDDEFAULTBRANCHNAMEREQUEST']._serialized_end=156
  _globals['_FINDDEFAULTBRANCHNAMERESPONSE']._serialized_start=158
  _globals['_FINDDEFAULTBRANCHNAMERESPONSE']._serialized_end=203
  _globals['_FINDLOCALBRANCHESREQUEST']._serialized_start=206
  _globals['_FINDLOCALBRANCHESREQUEST']._serialized_end=447
  _globals['_FINDLOCALBRANCHESREQUEST_SORTBY']._serialized_start=394
  _globals['_FINDLOCALBRANCHESREQUEST_SORTBY']._serialized_end=447
  _globals['_FINDLOCALBRANCHESRESPONSE']._serialized_start=449
  _globals['_FINDLOCALBRANCHESRESPONSE']._serialized_end=532
  _globals['_FINDALLBRANCHESREQUEST']._serialized_start=534
  _globals['_FINDALLBRANCHESREQUEST']._serialized_end=650
  _globals['_FINDALLBRANCHESRESPONSE']._serialized_start=653
  _globals['_FINDALLBRANCHESRESPONSE']._serialized_end=795
  _globals['_FINDALLBRANCHESRESPONSE_BRANCH']._serialized_start=738
  _globals['_FINDALLBRANCHESRESPONSE_BRANCH']._serialized_end=795
  _globals['_FINDTAGREQUEST']._serialized_start=797
  _globals['_FINDTAGREQUEST']._serialized_end=877
  _globals['_FINDTAGRESPONSE']._serialized_start=879
  _globals['_FINDTAGRESPONSE']._serialized_end=922
  _globals['_FINDTAGERROR']._serialized_start=924
  _globals['_FINDTAGERROR']._serialized_end=1004
  _globals['_FINDALLTAGSREQUEST']._serialized_start=1007
  _globals['_FINDALLTAGSREQUEST']._serialized_end=1344
  _globals['_FINDALLTAGSREQUEST_SORTBY']._serialized_start=1184
  _globals['_FINDALLTAGSREQUEST_SORTBY']._serialized_end=1344
  _globals['_FINDALLTAGSREQUEST_SORTBY_KEY']._serialized_start=1288
  _globals['_FINDALLTAGSREQUEST_SORTBY_KEY']._serialized_end=1344
  _globals['_FINDALLTAGSRESPONSE']._serialized_start=1346
  _globals['_FINDALLTAGSRESPONSE']._serialized_end=1394
  _globals['_REFEXISTSREQUEST']._serialized_start=1396
  _globals['_REFEXISTSREQUEST']._serialized_end=1473
  _globals['_REFEXISTSRESPONSE']._serialized_start=1475
  _globals['_REFEXISTSRESPONSE']._serialized_end=1509
  _globals['_FINDBRANCHREQUEST']._serialized_start=1511
  _globals['_FINDBRANCHREQUEST']._serialized_end=1590
  _globals['_FINDBRANCHRESPONSE']._serialized_start=1592
  _globals['_FINDBRANCHRESPONSE']._serialized_end=1644
  _globals['_UPDATEREFERENCESREQUEST']._serialized_start=1647
  _globals['_UPDATEREFERENCESREQUEST']._serialized_end=1850
  _globals['_UPDATEREFERENCESREQUEST_UPDATE']._serialized_start=1777
  _globals['_UPDATEREFERENCESREQUEST_UPDATE']._serialized_end=1850
  _globals['_UPDATEREFERENCESRESPONSE']._serialized_start=1852
  _globals['_UPDATEREFERENCESRESPONSE']._serialized_end=1878
  _globals['_UPDATEREFERENCESERROR']._serialized_start=1881
  _globals['_UPDATEREFERENCESERROR']._serialized_end=2103
  _globals['_DELETEREFSREQUEST']._serialized_start=2105
  _globals['_DELETEREFSREQUEST']._serialized_end=2212
  _globals['_DELETEREFSRESPONSE']._serialized_start=2214
  _globals['_DELETEREFSRESPONSE']._serialized_end=2253
  _globals['_DELETEREFSERROR']._serialized_start=2256
  _globals['_DELETEREFSERROR']._serialized_end=2399
  _globals['_LISTBRANCHNAMESCONTAININGCOMMITREQUEST']._serialized_start=2401
  _globals['_LISTBRANCHNAMESCONTAININGCOMMITREQUEST']._serialized_end=2521
  _globals['_LISTBRANCHNAMESCONTAININGCOMMITRESPONSE']._serialized_start=2523
  _globals['_LISTBRANCHNAMESCONTAININGCOMMITRESPONSE']._serialized_end=2592
  _globals['_LISTTAGNAMESCONTAININGCOMMITREQUEST']._serialized_start=2594
  _globals['_LISTTAGNAMESCONTAININGCOMMITREQUEST']._serialized_end=2711
  _globals['_LISTTAGNAMESCONTAININGCOMMITRESPONSE']._serialized_start=2713
  _globals['_LISTTAGNAMESCONTAININGCOMMITRESPONSE']._serialized_end=2776
  _globals['_GETTAGSIGNATURESREQUEST']._serialized_start=2778
  _globals['_GETTAGSIGNATURESREQUEST']._serialized_end=2872
  _globals['_GETTAGSIGNATURESRESPONSE']._serialized_start=2875
  _globals['_GETTAGSIGNATURESRESPONSE']._serialized_end=3036
  _globals['_GETTAGSIGNATURESRESPONSE_TAGSIGNATURE']._serialized_start=2970
  _globals['_GETTAGSIGNATURESRESPONSE_TAGSIGNATURE']._serialized_end=3036
  _globals['_GETTAGMESSAGESREQUEST']._serialized_start=3038
  _globals['_GETTAGMESSAGESREQUEST']._serialized_end=3141
  _globals['_GETTAGMESSAGESRESPONSE']._serialized_start=3143
  _globals['_GETTAGMESSAGESRESPONSE']._serialized_end=3216
  _globals['_FINDALLREMOTEBRANCHESREQUEST']._serialized_start=3218
  _globals['_FINDALLREMOTEBRANCHESREQUEST']._serialized_end=3315
  _globals['_FINDALLREMOTEBRANCHESRESPONSE']._serialized_start=3317
  _globals['_FINDALLREMOTEBRANCHESRESPONSE']._serialized_end=3382
  _globals['_LISTREFSREQUEST']._serialized_start=3385
  _globals['_LISTREFSREQUEST']._serialized_end=3748
  _globals['_LISTREFSREQUEST_SORTBY']._serialized_start=3577
  _globals['_LISTREFSREQUEST_SORTBY']._serialized_end=3748
  _globals['_LISTREFSREQUEST_SORTBY_KEY']._serialized_start=3678
  _globals['_LISTREFSREQUEST_SORTBY_KEY']._serialized_end=3748
  _globals['_LISTREFSRESPONSE']._serialized_start=3751
  _globals['_LISTREFSRESPONSE']._serialized_end=3891
  _globals['_LISTREFSRESPONSE_REFERENCE']._serialized_start=3827
  _globals['_LISTREFSRESPONSE_REFERENCE']._serialized_end=3891
  _globals['_FINDREFSBYOIDREQUEST']._serialized_start=3894
  _globals['_FINDREFSBYOIDREQUEST']._serialized_end=4032
  _globals['_FINDREFSBYOIDRESPONSE']._serialized_start=4034
  _globals['_FINDREFSBYOIDRESPONSE']._serialized_end=4071
  _globals['_REFSERVICE']._serialized_start=4074
  _globals['_REFSERVICE']._serialized_end=5601
# @@protoc_insertion_point(module_scope)
