# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: shared.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import lint_pb2 as lint__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cshared.proto\x12\x06gitaly\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\nlint.proto\"\xc3\x01\n\nRepository\x12\x14\n\x0cstorage_name\x18\x02 \x01(\t\x12\x15\n\rrelative_path\x18\x03 \x01(\t\x12\x1c\n\x14git_object_directory\x18\x04 \x01(\t\x12(\n git_alternate_object_directories\x18\x05 \x03(\t\x12\x15\n\rgl_repository\x18\x06 \x01(\t\x12\x17\n\x0fgl_project_path\x18\x08 \x01(\tJ\x04\x08\x01\x10\x02J\x04\x08\x07\x10\x08R\x04path\"+\n\rCommitTrailer\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x0c\"M\n\x0e\x43ommitStatInfo\x12\x11\n\tadditions\x18\x01 \x01(\x05\x12\x11\n\tdeletions\x18\x02 \x01(\x05\x12\x15\n\rchanged_files\x18\x03 \x01(\x05\"\xeb\x02\n\tGitCommit\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0f\n\x07subject\x18\x02 \x01(\x0c\x12\x0c\n\x04\x62ody\x18\x03 \x01(\x0c\x12$\n\x06\x61uthor\x18\x04 \x01(\x0b\x32\x14.gitaly.CommitAuthor\x12\'\n\tcommitter\x18\x05 \x01(\x0b\x32\x14.gitaly.CommitAuthor\x12\x12\n\nparent_ids\x18\x06 \x03(\t\x12\x11\n\tbody_size\x18\x07 \x01(\x03\x12-\n\x0esignature_type\x18\x08 \x01(\x0e\x32\x15.gitaly.SignatureType\x12\x0f\n\x07tree_id\x18\t \x01(\t\x12\'\n\x08trailers\x18\n \x03(\x0b\x32\x15.gitaly.CommitTrailer\x12+\n\x0bshort_stats\x18\x0b \x01(\x0b\x32\x16.gitaly.CommitStatInfo\x12\x15\n\rreferenced_by\x18\x0c \x03(\x0c\x12\x10\n\x08\x65ncoding\x18\r \x01(\t\"g\n\x0c\x43ommitAuthor\x12\x0c\n\x04name\x18\x01 \x01(\x0c\x12\r\n\x05\x65mail\x18\x02 \x01(\x0c\x12(\n\x04\x64\x61te\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08timezone\x18\x04 \x01(\x0c\"\x1b\n\nExitStatus\x12\r\n\x05value\x18\x01 \x01(\x05\"@\n\x06\x42ranch\x12\x0c\n\x04name\x18\x01 \x01(\x0c\x12(\n\rtarget_commit\x18\x02 \x01(\x0b\x32\x11.gitaly.GitCommit\"\xc5\x01\n\x03Tag\x12\x0c\n\x04name\x18\x01 \x01(\x0c\x12\n\n\x02id\x18\x02 \x01(\t\x12(\n\rtarget_commit\x18\x03 \x01(\x0b\x32\x11.gitaly.GitCommit\x12\x0f\n\x07message\x18\x04 \x01(\x0c\x12\x14\n\x0cmessage_size\x18\x05 \x01(\x03\x12$\n\x06tagger\x18\x06 \x01(\x0b\x32\x14.gitaly.CommitAuthor\x12-\n\x0esignature_type\x18\x07 \x01(\x0e\x32\x15.gitaly.SignatureType\"Y\n\x04User\x12\r\n\x05gl_id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\x0c\x12\r\n\x05\x65mail\x18\x03 \x01(\x0c\x12\x13\n\x0bgl_username\x18\x04 \x01(\t\x12\x10\n\x08timezone\x18\x05 \x01(\t\":\n\nObjectPool\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x90\xc6,\x01\"8\n\x13PaginationParameter\x12\x12\n\npage_token\x18\x01 \x01(\t\x12\r\n\x05limit\x18\x02 \x01(\x05\"\'\n\x10PaginationCursor\x12\x13\n\x0bnext_cursor\x18\x01 \x01(\t\"*\n\rGlobalOptions\x12\x19\n\x11literal_pathspecs\x18\x01 \x01(\x08*B\n\nObjectType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06\x43OMMIT\x10\x01\x12\x08\n\x04\x42LOB\x10\x02\x12\x08\n\x04TREE\x10\x03\x12\x07\n\x03TAG\x10\x04*_\n\x0cObjectFormat\x12\x1d\n\x19OBJECT_FORMAT_UNSPECIFIED\x10\x00\x12\x16\n\x12OBJECT_FORMAT_SHA1\x10\x01\x12\x18\n\x14OBJECT_FORMAT_SHA256\x10\x02*5\n\rSignatureType\x12\x08\n\x04NONE\x10\x00\x12\x07\n\x03PGP\x10\x01\x12\x08\n\x04X509\x10\x02\x12\x07\n\x03SSH\x10\x03*.\n\rSortDirection\x12\r\n\tASCENDING\x10\x00\x12\x0e\n\nDESCENDING\x10\x01\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'shared_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _OBJECTPOOL.fields_by_name['repository']._options = None
  _OBJECTPOOL.fields_by_name['repository']._serialized_options = b'\220\306,\001'
  _globals['_OBJECTTYPE']._serialized_start=1451
  _globals['_OBJECTTYPE']._serialized_end=1517
  _globals['_OBJECTFORMAT']._serialized_start=1519
  _globals['_OBJECTFORMAT']._serialized_end=1614
  _globals['_SIGNATURETYPE']._serialized_start=1616
  _globals['_SIGNATURETYPE']._serialized_end=1669
  _globals['_SORTDIRECTION']._serialized_start=1671
  _globals['_SORTDIRECTION']._serialized_end=1717
  _globals['_REPOSITORY']._serialized_start=70
  _globals['_REPOSITORY']._serialized_end=265
  _globals['_COMMITTRAILER']._serialized_start=267
  _globals['_COMMITTRAILER']._serialized_end=310
  _globals['_COMMITSTATINFO']._serialized_start=312
  _globals['_COMMITSTATINFO']._serialized_end=389
  _globals['_GITCOMMIT']._serialized_start=392
  _globals['_GITCOMMIT']._serialized_end=755
  _globals['_COMMITAUTHOR']._serialized_start=757
  _globals['_COMMITAUTHOR']._serialized_end=860
  _globals['_EXITSTATUS']._serialized_start=862
  _globals['_EXITSTATUS']._serialized_end=889
  _globals['_BRANCH']._serialized_start=891
  _globals['_BRANCH']._serialized_end=955
  _globals['_TAG']._serialized_start=958
  _globals['_TAG']._serialized_end=1155
  _globals['_USER']._serialized_start=1157
  _globals['_USER']._serialized_end=1246
  _globals['_OBJECTPOOL']._serialized_start=1248
  _globals['_OBJECTPOOL']._serialized_end=1306
  _globals['_PAGINATIONPARAMETER']._serialized_start=1308
  _globals['_PAGINATIONPARAMETER']._serialized_end=1364
  _globals['_PAGINATIONCURSOR']._serialized_start=1366
  _globals['_PAGINATIONCURSOR']._serialized_end=1405
  _globals['_GLOBALOPTIONS']._serialized_start=1407
  _globals['_GLOBALOPTIONS']._serialized_end=1449
# @@protoc_insertion_point(module_scope)
