# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: analysis.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x61nalysis.proto\x12\x06gitaly\x1a\nlint.proto\x1a\x0cshared.proto\"\xaa\x01\n\x1a\x43heckBlobsGeneratedRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x36\n\x05\x62lobs\x18\x02 \x03(\x0b\x32\'.gitaly.CheckBlobsGeneratedRequest.Blob\x1a&\n\x04\x42lob\x12\x10\n\x08revision\x18\x01 \x01(\x0c\x12\x0c\n\x04path\x18\x02 \x01(\x0c\"\x83\x01\n\x1b\x43heckBlobsGeneratedResponse\x12\x37\n\x05\x62lobs\x18\x01 \x03(\x0b\x32(.gitaly.CheckBlobsGeneratedResponse.Blob\x1a+\n\x04\x42lob\x12\x10\n\x08revision\x18\x01 \x01(\x0c\x12\x11\n\tgenerated\x18\x02 \x01(\x08\x32}\n\x0f\x41nalysisService\x12j\n\x13\x43heckBlobsGenerated\x12\".gitaly.CheckBlobsGeneratedRequest\x1a#.gitaly.CheckBlobsGeneratedResponse\"\x06\xfa\x97(\x02\x08\x02(\x01\x30\x01\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'analysis_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _CHECKBLOBSGENERATEDREQUEST.fields_by_name['repository']._options = None
  _CHECKBLOBSGENERATEDREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _ANALYSISSERVICE.methods_by_name['CheckBlobsGenerated']._options = None
  _ANALYSISSERVICE.methods_by_name['CheckBlobsGenerated']._serialized_options = b'\372\227(\002\010\002'
  _globals['_CHECKBLOBSGENERATEDREQUEST']._serialized_start=53
  _globals['_CHECKBLOBSGENERATEDREQUEST']._serialized_end=223
  _globals['_CHECKBLOBSGENERATEDREQUEST_BLOB']._serialized_start=185
  _globals['_CHECKBLOBSGENERATEDREQUEST_BLOB']._serialized_end=223
  _globals['_CHECKBLOBSGENERATEDRESPONSE']._serialized_start=226
  _globals['_CHECKBLOBSGENERATEDRESPONSE']._serialized_end=357
  _globals['_CHECKBLOBSGENERATEDRESPONSE_BLOB']._serialized_start=314
  _globals['_CHECKBLOBSGENERATEDRESPONSE_BLOB']._serialized_end=357
  _globals['_ANALYSISSERVICE']._serialized_start=359
  _globals['_ANALYSISSERVICE']._serialized_end=484
# @@protoc_insertion_point(module_scope)
