# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mercurial-repository.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1amercurial-repository.proto\x12\x07hgitaly\x1a\nlint.proto\x1a\x0cshared.proto\"Y\n\x11InitConfigRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x16\n\x0enamespace_path\x18\x02 \x01(\t\"\x14\n\x12InitConfigResponse\"\x95\x02\n\x15HeptapodConfigSection\x12!\n\x14\x61llow_multiple_heads\x18\x02 \x01(\x08H\x00\x88\x01\x01\x12\x1c\n\x0f\x61llow_bookmarks\x18\x03 \x01(\x08H\x01\x88\x01\x01\x12\x45\n\x0c\x61uto_publish\x18\x04 \x01(\x0e\x32*.hgitaly.HeptapodConfigSection.AutoPublishH\x02\x88\x01\x01\"6\n\x0b\x41utoPublish\x12\x11\n\rWITHOUT_TOPIC\x10\x00\x12\x0b\n\x07NOTHING\x10\x01\x12\x07\n\x03\x41LL\x10\x02\x42\x17\n\x15_allow_multiple_headsB\x12\n\x10_allow_bookmarksB\x0f\n\r_auto_publish\"V\n\x17GetManagedConfigRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\r\n\x05local\x18\x02 \x01(\x08\"]\n\x18GetManagedConfigResponse\x12\x0f\n\x07inherit\x18\x01 \x01(\x08\x12\x30\n\x08heptapod\x18\x02 \x01(\x0b\x32\x1e.hgitaly.HeptapodConfigSection\"\xc2\x01\n\x17SetManagedConfigRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x14\n\x07inherit\x18\x02 \x01(\x08H\x00\x88\x01\x01\x12\x30\n\x08heptapod\x18\x03 \x01(\x0b\x32\x1e.hgitaly.HeptapodConfigSection\x12\x14\n\x0cremove_items\x18\x04 \x03(\t\x12\x0f\n\x07\x62y_line\x18\x05 \x01(\tB\n\n\x08_inherit\"\x1a\n\x18SetManagedConfigResponse\"\x8d\x01\n\x14GetConfigItemRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12(\n\x07\x61s_type\x18\x02 \x01(\x0e\x32\x17.hgitaly.ConfigItemType\x12\x0f\n\x07section\x18\x03 \x01(\t\x12\x0c\n\x04name\x18\x04 \x01(\t\"H\n\x15GetConfigItemResponse\x12\x13\n\tas_string\x18\x01 \x01(\tH\x00\x12\x11\n\x07\x61s_bool\x18\x02 \x01(\x08H\x00\x42\x07\n\x05value\">\n\x0eRecoverRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"%\n\x0fRecoverResponse\x12\x12\n\nnot_needed\x18\x01 \x01(\x08\"?\n\x0fOptimizeRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\x12\n\x10OptimizeResponse\"\x88\x01\n\x12ResetCachesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x31\n\x06\x63\x61\x63hes\x18\x02 \x03(\x0e\x32!.hgitaly.ResetCachesRequest.Cache\"\x11\n\x05\x43\x61\x63he\x12\x08\n\x04TAGS\x10\x00\"\x15\n\x13ResetCachesResponse\"b\n\rMercurialPeer\x12\x0b\n\x03url\x18\x01 \x01(\t\x12\x0f\n\x07ssh_key\x18\x05 \x01(\t\x12\x17\n\x0fssh_known_hosts\x18\x06 \x01(\t\x12\x1a\n\x12ssh_remote_command\x18\x07 \x01(\x0c\"\xa7\x01\n\x0bPushRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12+\n\x0bremote_peer\x18\x02 \x01(\x0b\x32\x16.hgitaly.MercurialPeer\x12%\n\x1donly_gitlab_branches_matching\x18\x03 \x03(\x0c\x12\x16\n\x0einclude_drafts\x18\x04 \x01(\x08\"&\n\x0cPushResponse\x12\x16\n\x0enew_changesets\x18\x01 \x01(\x08\"\xb8\x01\n\x0bPullRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12+\n\x0bremote_peer\x18\x02 \x01(\x0b\x32\x16.hgitaly.MercurialPeer\x12\x1a\n\x04user\x18\x04 \x01(\x0b\x32\x0c.gitaly.User\x12\x1b\n\x13mercurial_revisions\x18\x05 \x03(\x0cJ\x04\x08\x03\x10\x04R\x0fgitlab_branches\"&\n\x0cPullResponse\x12\x16\n\x0enew_changesets\x18\x01 \x01(\x08*&\n\x0e\x43onfigItemType\x12\n\n\x06STRING\x10\x00\x12\x08\n\x04\x42OOL\x10\x01\x32\xe0\x05\n\x1aMercurialRepositoryService\x12M\n\nInitConfig\x12\x1a.hgitaly.InitConfigRequest\x1a\x1b.hgitaly.InitConfigResponse\"\x06\xfa\x97(\x02\x08\x01\x12V\n\rGetConfigItem\x12\x1d.hgitaly.GetConfigItemRequest\x1a\x1e.hgitaly.GetConfigItemResponse\"\x06\xfa\x97(\x02\x08\x02\x12\x44\n\x07Recover\x12\x17.hgitaly.RecoverRequest\x1a\x18.hgitaly.RecoverResponse\"\x06\xfa\x97(\x02\x08\x02\x12G\n\x08Optimize\x12\x18.hgitaly.OptimizeRequest\x1a\x19.hgitaly.OptimizeResponse\"\x06\xfa\x97(\x02\x08\x01\x12P\n\x0bResetCaches\x12\x1b.hgitaly.ResetCachesRequest\x1a\x1c.hgitaly.ResetCachesResponse\"\x06\xfa\x97(\x02\x08\x01\x12_\n\x10GetManagedConfig\x12 .hgitaly.GetManagedConfigRequest\x1a!.hgitaly.GetManagedConfigResponse\"\x06\xfa\x97(\x02\x08\x02\x12_\n\x10SetManagedConfig\x12 .hgitaly.SetManagedConfigRequest\x1a!.hgitaly.SetManagedConfigResponse\"\x06\xfa\x97(\x02\x08\x01\x12;\n\x04Push\x12\x14.hgitaly.PushRequest\x1a\x15.hgitaly.PushResponse\"\x06\xfa\x97(\x02\x08\x01\x12;\n\x04Pull\x12\x14.hgitaly.PullRequest\x1a\x15.hgitaly.PullResponse\"\x06\xfa\x97(\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mercurial_repository_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _INITCONFIGREQUEST.fields_by_name['repository']._options = None
  _INITCONFIGREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETMANAGEDCONFIGREQUEST.fields_by_name['repository']._options = None
  _GETMANAGEDCONFIGREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _SETMANAGEDCONFIGREQUEST.fields_by_name['repository']._options = None
  _SETMANAGEDCONFIGREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETCONFIGITEMREQUEST.fields_by_name['repository']._options = None
  _GETCONFIGITEMREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _RECOVERREQUEST.fields_by_name['repository']._options = None
  _RECOVERREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _OPTIMIZEREQUEST.fields_by_name['repository']._options = None
  _OPTIMIZEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _RESETCACHESREQUEST.fields_by_name['repository']._options = None
  _RESETCACHESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _PUSHREQUEST.fields_by_name['repository']._options = None
  _PUSHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _PULLREQUEST.fields_by_name['repository']._options = None
  _PULLREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _MERCURIALREPOSITORYSERVICE.methods_by_name['InitConfig']._options = None
  _MERCURIALREPOSITORYSERVICE.methods_by_name['InitConfig']._serialized_options = b'\372\227(\002\010\001'
  _MERCURIALREPOSITORYSERVICE.methods_by_name['GetConfigItem']._options = None
  _MERCURIALREPOSITORYSERVICE.methods_by_name['GetConfigItem']._serialized_options = b'\372\227(\002\010\002'
  _MERCURIALREPOSITORYSERVICE.methods_by_name['Recover']._options = None
  _MERCURIALREPOSITORYSERVICE.methods_by_name['Recover']._serialized_options = b'\372\227(\002\010\002'
  _MERCURIALREPOSITORYSERVICE.methods_by_name['Optimize']._options = None
  _MERCURIALREPOSITORYSERVICE.methods_by_name['Optimize']._serialized_options = b'\372\227(\002\010\001'
  _MERCURIALREPOSITORYSERVICE.methods_by_name['ResetCaches']._options = None
  _MERCURIALREPOSITORYSERVICE.methods_by_name['ResetCaches']._serialized_options = b'\372\227(\002\010\001'
  _MERCURIALREPOSITORYSERVICE.methods_by_name['GetManagedConfig']._options = None
  _MERCURIALREPOSITORYSERVICE.methods_by_name['GetManagedConfig']._serialized_options = b'\372\227(\002\010\002'
  _MERCURIALREPOSITORYSERVICE.methods_by_name['SetManagedConfig']._options = None
  _MERCURIALREPOSITORYSERVICE.methods_by_name['SetManagedConfig']._serialized_options = b'\372\227(\002\010\001'
  _MERCURIALREPOSITORYSERVICE.methods_by_name['Push']._options = None
  _MERCURIALREPOSITORYSERVICE.methods_by_name['Push']._serialized_options = b'\372\227(\002\010\001'
  _MERCURIALREPOSITORYSERVICE.methods_by_name['Pull']._options = None
  _MERCURIALREPOSITORYSERVICE.methods_by_name['Pull']._serialized_options = b'\372\227(\002\010\001'
  _globals['_CONFIGITEMTYPE']._serialized_start=1971
  _globals['_CONFIGITEMTYPE']._serialized_end=2009
  _globals['_INITCONFIGREQUEST']._serialized_start=65
  _globals['_INITCONFIGREQUEST']._serialized_end=154
  _globals['_INITCONFIGRESPONSE']._serialized_start=156
  _globals['_INITCONFIGRESPONSE']._serialized_end=176
  _globals['_HEPTAPODCONFIGSECTION']._serialized_start=179
  _globals['_HEPTAPODCONFIGSECTION']._serialized_end=456
  _globals['_HEPTAPODCONFIGSECTION_AUTOPUBLISH']._serialized_start=340
  _globals['_HEPTAPODCONFIGSECTION_AUTOPUBLISH']._serialized_end=394
  _globals['_GETMANAGEDCONFIGREQUEST']._serialized_start=458
  _globals['_GETMANAGEDCONFIGREQUEST']._serialized_end=544
  _globals['_GETMANAGEDCONFIGRESPONSE']._serialized_start=546
  _globals['_GETMANAGEDCONFIGRESPONSE']._serialized_end=639
  _globals['_SETMANAGEDCONFIGREQUEST']._serialized_start=642
  _globals['_SETMANAGEDCONFIGREQUEST']._serialized_end=836
  _globals['_SETMANAGEDCONFIGRESPONSE']._serialized_start=838
  _globals['_SETMANAGEDCONFIGRESPONSE']._serialized_end=864
  _globals['_GETCONFIGITEMREQUEST']._serialized_start=867
  _globals['_GETCONFIGITEMREQUEST']._serialized_end=1008
  _globals['_GETCONFIGITEMRESPONSE']._serialized_start=1010
  _globals['_GETCONFIGITEMRESPONSE']._serialized_end=1082
  _globals['_RECOVERREQUEST']._serialized_start=1084
  _globals['_RECOVERREQUEST']._serialized_end=1146
  _globals['_RECOVERRESPONSE']._serialized_start=1148
  _globals['_RECOVERRESPONSE']._serialized_end=1185
  _globals['_OPTIMIZEREQUEST']._serialized_start=1187
  _globals['_OPTIMIZEREQUEST']._serialized_end=1250
  _globals['_OPTIMIZERESPONSE']._serialized_start=1252
  _globals['_OPTIMIZERESPONSE']._serialized_end=1270
  _globals['_RESETCACHESREQUEST']._serialized_start=1273
  _globals['_RESETCACHESREQUEST']._serialized_end=1409
  _globals['_RESETCACHESREQUEST_CACHE']._serialized_start=1392
  _globals['_RESETCACHESREQUEST_CACHE']._serialized_end=1409
  _globals['_RESETCACHESRESPONSE']._serialized_start=1411
  _globals['_RESETCACHESRESPONSE']._serialized_end=1432
  _globals['_MERCURIALPEER']._serialized_start=1434
  _globals['_MERCURIALPEER']._serialized_end=1532
  _globals['_PUSHREQUEST']._serialized_start=1535
  _globals['_PUSHREQUEST']._serialized_end=1702
  _globals['_PUSHRESPONSE']._serialized_start=1704
  _globals['_PUSHRESPONSE']._serialized_end=1742
  _globals['_PULLREQUEST']._serialized_start=1745
  _globals['_PULLREQUEST']._serialized_end=1929
  _globals['_PULLRESPONSE']._serialized_start=1931
  _globals['_PULLRESPONSE']._serialized_end=1969
  _globals['_MERCURIALREPOSITORYSERVICE']._serialized_start=2012
  _globals['_MERCURIALREPOSITORYSERVICE']._serialized_end=2748
# @@protoc_insertion_point(module_scope)
