# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: server.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from . import lint_pb2 as lint__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cserver.proto\x12\x06gitaly\x1a\x1egoogle/protobuf/duration.proto\x1a\nlint.proto\"\x13\n\x11ServerInfoRequest\"\x96\x02\n\x12ServerInfoResponse\x12\x16\n\x0eserver_version\x18\x01 \x01(\t\x12\x13\n\x0bgit_version\x18\x02 \x01(\t\x12\x42\n\x10storage_statuses\x18\x03 \x03(\x0b\x32(.gitaly.ServerInfoResponse.StorageStatus\x1a\x8e\x01\n\rStorageStatus\x12\x14\n\x0cstorage_name\x18\x01 \x01(\t\x12\x10\n\x08readable\x18\x02 \x01(\x08\x12\x11\n\twriteable\x18\x03 \x01(\x08\x12\x0f\n\x07\x66s_type\x18\x04 \x01(\t\x12\x15\n\rfilesystem_id\x18\x05 \x01(\t\x12\x1a\n\x12replication_factor\x18\x06 \x01(\r\"\x17\n\x15\x44iskStatisticsRequest\"\xa8\x01\n\x16\x44iskStatisticsResponse\x12\x46\n\x10storage_statuses\x18\x01 \x03(\x0b\x32,.gitaly.DiskStatisticsResponse.StorageStatus\x1a\x46\n\rStorageStatus\x12\x14\n\x0cstorage_name\x18\x01 \x01(\t\x12\x11\n\tavailable\x18\x02 \x01(\x03\x12\x0c\n\x04used\x18\x03 \x01(\x03\"x\n\x12\x43lockSyncedRequest\x12\x10\n\x08ntp_host\x18\x01 \x01(\t\x12\x32\n\x0f\x64rift_threshold\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationJ\x04\x08\x02\x10\x03R\x16\x64rift_threshold_millis\"%\n\x13\x43lockSyncedResponse\x12\x0e\n\x06synced\x18\x01 \x01(\x08\"C\n\x15ReadinessCheckRequest\x12*\n\x07timeout\x18\x01 \x01(\x0b\x32\x19.google.protobuf.Duration\"\xab\x02\n\x16ReadinessCheckResponse\x12\x38\n\x0bok_response\x18\x01 \x01(\x0b\x32!.gitaly.ReadinessCheckResponse.OkH\x00\x12\x42\n\x10\x66\x61ilure_response\x18\x02 \x01(\x0b\x32&.gitaly.ReadinessCheckResponse.FailureH\x00\x1a\x04\n\x02Ok\x1a\x82\x01\n\x07\x46\x61ilure\x12\x46\n\rfailed_checks\x18\x01 \x03(\x0b\x32/.gitaly.ReadinessCheckResponse.Failure.Response\x1a/\n\x08Response\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rerror_message\x18\x02 \x01(\tB\x08\n\x06Result2\xc4\x02\n\rServerService\x12\x43\n\nServerInfo\x12\x19.gitaly.ServerInfoRequest\x1a\x1a.gitaly.ServerInfoResponse\x12O\n\x0e\x44iskStatistics\x12\x1d.gitaly.DiskStatisticsRequest\x1a\x1e.gitaly.DiskStatisticsResponse\x12\x46\n\x0b\x43lockSynced\x12\x1a.gitaly.ClockSyncedRequest\x1a\x1b.gitaly.ClockSyncedResponse\x12O\n\x0eReadinessCheck\x12\x1d.gitaly.ReadinessCheckRequest\x1a\x1e.gitaly.ReadinessCheckResponse\x1a\x04\xf0\x97(\x01\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'server_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _SERVERSERVICE._options = None
  _SERVERSERVICE._serialized_options = b'\360\227(\001'
  _globals['_SERVERINFOREQUEST']._serialized_start=68
  _globals['_SERVERINFOREQUEST']._serialized_end=87
  _globals['_SERVERINFORESPONSE']._serialized_start=90
  _globals['_SERVERINFORESPONSE']._serialized_end=368
  _globals['_SERVERINFORESPONSE_STORAGESTATUS']._serialized_start=226
  _globals['_SERVERINFORESPONSE_STORAGESTATUS']._serialized_end=368
  _globals['_DISKSTATISTICSREQUEST']._serialized_start=370
  _globals['_DISKSTATISTICSREQUEST']._serialized_end=393
  _globals['_DISKSTATISTICSRESPONSE']._serialized_start=396
  _globals['_DISKSTATISTICSRESPONSE']._serialized_end=564
  _globals['_DISKSTATISTICSRESPONSE_STORAGESTATUS']._serialized_start=494
  _globals['_DISKSTATISTICSRESPONSE_STORAGESTATUS']._serialized_end=564
  _globals['_CLOCKSYNCEDREQUEST']._serialized_start=566
  _globals['_CLOCKSYNCEDREQUEST']._serialized_end=686
  _globals['_CLOCKSYNCEDRESPONSE']._serialized_start=688
  _globals['_CLOCKSYNCEDRESPONSE']._serialized_end=725
  _globals['_READINESSCHECKREQUEST']._serialized_start=727
  _globals['_READINESSCHECKREQUEST']._serialized_end=794
  _globals['_READINESSCHECKRESPONSE']._serialized_start=797
  _globals['_READINESSCHECKRESPONSE']._serialized_end=1096
  _globals['_READINESSCHECKRESPONSE_OK']._serialized_start=949
  _globals['_READINESSCHECKRESPONSE_OK']._serialized_end=953
  _globals['_READINESSCHECKRESPONSE_FAILURE']._serialized_start=956
  _globals['_READINESSCHECKRESPONSE_FAILURE']._serialized_end=1086
  _globals['_READINESSCHECKRESPONSE_FAILURE_RESPONSE']._serialized_start=1039
  _globals['_READINESSCHECKRESPONSE_FAILURE_RESPONSE']._serialized_end=1086
  _globals['_SERVERSERVICE']._serialized_start=1099
  _globals['_SERVERSERVICE']._serialized_end=1423
# @@protoc_insertion_point(module_scope)
