# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: commit.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import errors_pb2 as errors__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x63ommit.proto\x12\x06gitaly\x1a\x0c\x65rrors.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\nlint.proto\x1a\x0cshared.proto\"\xd4\x03\n\x12ListCommitsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\trevisions\x18\x02 \x03(\t\x12\x36\n\x11pagination_params\x18\x03 \x01(\x0b\x32\x1b.gitaly.PaginationParameter\x12/\n\x05order\x18\x04 \x01(\x0e\x32 .gitaly.ListCommitsRequest.Order\x12\x0f\n\x07reverse\x18\x0b \x01(\x08\x12\x13\n\x0bmax_parents\x18\x05 \x01(\r\x12\x14\n\x0c\x64isable_walk\x18\x06 \x01(\x08\x12\x14\n\x0c\x66irst_parent\x18\x07 \x01(\x08\x12)\n\x05\x61\x66ter\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12*\n\x06\x62\x65\x66ore\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06\x61uthor\x18\n \x01(\x0c\x12\x13\n\x0bignore_case\x18\x0c \x01(\x08\x12\x1f\n\x17\x63ommit_message_patterns\x18\r \x03(\x0c\"%\n\x05Order\x12\x08\n\x04NONE\x10\x00\x12\x08\n\x04TOPO\x10\x01\x12\x08\n\x04\x44\x41TE\x10\x02\"9\n\x13ListCommitsResponse\x12\"\n\x07\x63ommits\x18\x01 \x03(\x0b\x32\x11.gitaly.GitCommit\"}\n\x15ListAllCommitsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x36\n\x11pagination_params\x18\x02 \x01(\x0b\x32\x1b.gitaly.PaginationParameter\"<\n\x16ListAllCommitsResponse\x12\"\n\x07\x63ommits\x18\x01 \x03(\x0b\x32\x11.gitaly.GitCommit\"T\n\x12\x43ommitStatsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\"H\n\x13\x43ommitStatsResponse\x12\x0b\n\x03oid\x18\x01 \x01(\t\x12\x11\n\tadditions\x18\x02 \x01(\x05\x12\x11\n\tdeletions\x18\x03 \x01(\x05\"n\n\x17\x43ommitIsAncestorRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x13\n\x0b\x61ncestor_id\x18\x02 \x01(\t\x12\x10\n\x08\x63hild_id\x18\x03 \x01(\t\")\n\x18\x43ommitIsAncestorResponse\x12\r\n\x05value\x18\x01 \x01(\x08\"\x81\x01\n\x10TreeEntryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\x12\x0c\n\x04path\x18\x03 \x01(\x0c\x12\r\n\x05limit\x18\x04 \x01(\x03\x12\x10\n\x08max_size\x18\x05 \x01(\x03\"\xb5\x01\n\x11TreeEntryResponse\x12\x32\n\x04type\x18\x01 \x01(\x0e\x32$.gitaly.TreeEntryResponse.ObjectType\x12\x0b\n\x03oid\x18\x02 \x01(\t\x12\x0c\n\x04size\x18\x03 \x01(\x03\x12\x0c\n\x04mode\x18\x04 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x05 \x01(\x0c\"5\n\nObjectType\x12\n\n\x06\x43OMMIT\x10\x00\x12\x08\n\x04\x42LOB\x10\x01\x12\x08\n\x04TREE\x10\x02\x12\x07\n\x03TAG\x10\x03\"\x9f\x02\n\x13\x43ountCommitsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\x12)\n\x05\x61\x66ter\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12*\n\x06\x62\x65\x66ore\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0c\n\x04path\x18\x05 \x01(\x0c\x12\x11\n\tmax_count\x18\x06 \x01(\x05\x12\x0b\n\x03\x61ll\x18\x07 \x01(\x08\x12\x14\n\x0c\x66irst_parent\x18\x08 \x01(\x08\x12-\n\x0eglobal_options\x18\t \x01(\x0b\x32\x15.gitaly.GlobalOptions\"%\n\x14\x43ountCommitsResponse\x12\r\n\x05\x63ount\x18\x01 \x01(\x05\"\x8b\x01\n\x1c\x43ountDivergingCommitsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0c\n\x04\x66rom\x18\x02 \x01(\x0c\x12\n\n\x02to\x18\x03 \x01(\x0c\x12\x11\n\tmax_count\x18\x07 \x01(\x05J\x04\x08\x04\x10\x05J\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07\"H\n\x1d\x43ountDivergingCommitsResponse\x12\x12\n\nleft_count\x18\x01 \x01(\x05\x12\x13\n\x0bright_count\x18\x02 \x01(\x05\"\xc3\x01\n\tTreeEntry\x12\x0b\n\x03oid\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x03 \x01(\x0c\x12)\n\x04type\x18\x04 \x01(\x0e\x32\x1b.gitaly.TreeEntry.EntryType\x12\x0c\n\x04mode\x18\x05 \x01(\x05\x12\x12\n\ncommit_oid\x18\x06 \x01(\t\x12\x11\n\tflat_path\x18\x07 \x01(\x0c\"+\n\tEntryType\x12\x08\n\x04\x42LOB\x10\x00\x12\x08\n\x04TREE\x10\x01\x12\n\n\x06\x43OMMIT\x10\x03J\x04\x08\x02\x10\x03R\x08root_oid\"\xa5\x02\n\x15GetTreeEntriesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\x12\x0c\n\x04path\x18\x03 \x01(\x0c\x12\x11\n\trecursive\x18\x04 \x01(\x08\x12\x32\n\x04sort\x18\x05 \x01(\x0e\x32$.gitaly.GetTreeEntriesRequest.SortBy\x12\x36\n\x11pagination_params\x18\x06 \x01(\x0b\x32\x1b.gitaly.PaginationParameter\x12\x17\n\x0fskip_flat_paths\x18\x07 \x01(\x08\"&\n\x06SortBy\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x0f\n\x0bTREES_FIRST\x10\x01\"q\n\x16GetTreeEntriesResponse\x12\"\n\x07\x65ntries\x18\x01 \x03(\x0b\x32\x11.gitaly.TreeEntry\x12\x33\n\x11pagination_cursor\x18\x02 \x01(\x0b\x32\x18.gitaly.PaginationCursor\"w\n\x13GetTreeEntriesError\x12\x34\n\x0cresolve_tree\x18\x01 \x01(\x0b\x32\x1c.gitaly.ResolveRevisionErrorH\x00\x12!\n\x04path\x18\x02 \x01(\x0b\x32\x11.gitaly.PathErrorH\x00\x42\x07\n\x05\x65rror\"R\n\x10ListFilesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\"\"\n\x11ListFilesResponse\x12\r\n\x05paths\x18\x01 \x03(\x0c\"e\n\x11\x46indCommitRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\x12\x10\n\x08trailers\x18\x03 \x01(\x08\"7\n\x12\x46indCommitResponse\x12!\n\x06\x63ommit\x18\x01 \x01(\x0b\x32\x11.gitaly.GitCommit\"T\n\x17ListCommitsByOidRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0b\n\x03oid\x18\x02 \x03(\t\">\n\x18ListCommitsByOidResponse\x12\"\n\x07\x63ommits\x18\x01 \x03(\x0b\x32\x11.gitaly.GitCommit\"^\n\x1bListCommitsByRefNameRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\tref_names\x18\x02 \x03(\x0c\"\xb1\x01\n\x1cListCommitsByRefNameResponse\x12\x46\n\x0b\x63ommit_refs\x18\x02 \x03(\x0b\x32\x31.gitaly.ListCommitsByRefNameResponse.CommitForRef\x1a\x43\n\x0c\x43ommitForRef\x12!\n\x06\x63ommit\x18\x01 \x01(\x0b\x32\x11.gitaly.GitCommit\x12\x10\n\x08ref_name\x18\x02 \x01(\x0cJ\x04\x08\x01\x10\x02\"\xd3\x01\n\x15\x46indAllCommitsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\x12\x11\n\tmax_count\x18\x03 \x01(\x05\x12\x0c\n\x04skip\x18\x04 \x01(\x05\x12\x32\n\x05order\x18\x05 \x01(\x0e\x32#.gitaly.FindAllCommitsRequest.Order\"%\n\x05Order\x12\x08\n\x04NONE\x10\x00\x12\x08\n\x04TOPO\x10\x01\x12\x08\n\x04\x44\x41TE\x10\x02\"<\n\x16\x46indAllCommitsResponse\x12\"\n\x07\x63ommits\x18\x01 \x03(\x0b\x32\x11.gitaly.GitCommit\"\x94\x04\n\x12\x46indCommitsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\x12\r\n\x05limit\x18\x03 \x01(\x05\x12\x0e\n\x06offset\x18\x04 \x01(\x05\x12\r\n\x05paths\x18\x05 \x03(\x0c\x12\x0e\n\x06\x66ollow\x18\x06 \x01(\x08\x12\x13\n\x0bskip_merges\x18\x07 \x01(\x08\x12\x18\n\x0c\x64isable_walk\x18\x08 \x01(\x08\x42\x02\x18\x01\x12)\n\x05\x61\x66ter\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12*\n\x06\x62\x65\x66ore\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0b\n\x03\x61ll\x18\x0b \x01(\x08\x12\x14\n\x0c\x66irst_parent\x18\x0c \x01(\x08\x12\x0e\n\x06\x61uthor\x18\r \x01(\x0c\x12/\n\x05order\x18\x0e \x01(\x0e\x32 .gitaly.FindCommitsRequest.Order\x12-\n\x0eglobal_options\x18\x0f \x01(\x0b\x32\x15.gitaly.GlobalOptions\x12\x10\n\x08trailers\x18\x10 \x01(\x08\x12\x19\n\x11include_shortstat\x18\x11 \x01(\x08\x12\x1d\n\x15include_referenced_by\x18\x12 \x03(\x0c\"\x1b\n\x05Order\x12\x08\n\x04NONE\x10\x00\x12\x08\n\x04TOPO\x10\x01\"9\n\x13\x46indCommitsResponse\x12\"\n\x07\x63ommits\x18\x01 \x03(\x0b\x32\x11.gitaly.GitCommit\"\xba\x01\n\x10\x46indCommitsError\x12,\n\nbad_object\x18\x01 \x01(\x0b\x32\x16.gitaly.BadObjectErrorH\x00\x12\x38\n\rambiguous_ref\x18\x02 \x01(\x0b\x32\x1f.gitaly.AmbiguousReferenceErrorH\x00\x12\x35\n\rinvalid_range\x18\x03 \x01(\x0b\x32\x1c.gitaly.InvalidRevisionRangeH\x00\x42\x07\n\x05\x65rror\"X\n\x16\x43ommitLanguagesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\"\xaf\x01\n\x17\x43ommitLanguagesResponse\x12;\n\tlanguages\x18\x01 \x03(\x0b\x32(.gitaly.CommitLanguagesResponse.Language\x1aW\n\x08Language\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05share\x18\x02 \x01(\x02\x12\r\n\x05\x63olor\x18\x03 \x01(\t\x12\r\n\x05\x62ytes\x18\x05 \x01(\x04J\x04\x08\x04\x10\x05R\nfile_count\"n\n\x0fRawBlameRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\x12\x0c\n\x04path\x18\x03 \x01(\x0c\x12\r\n\x05range\x18\x04 \x01(\x0c\" \n\x10RawBlameResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\xb5\x01\n\rRawBlameError\x12\x33\n\x0epath_not_found\x18\x01 \x01(\x0b\x32\x19.gitaly.PathNotFoundErrorH\x00\x12=\n\x0cout_of_range\x18\x02 \x01(\x0b\x32%.gitaly.RawBlameError.OutOfRangeErrorH\x00\x1a\'\n\x0fOutOfRangeError\x12\x14\n\x0c\x61\x63tual_lines\x18\x01 \x01(\x04\x42\x07\n\x05\x65rror\"\xb1\x01\n\x18LastCommitForPathRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\x12\x0c\n\x04path\x18\x03 \x01(\x0c\x12\x18\n\x10literal_pathspec\x18\x04 \x01(\x08\x12-\n\x0eglobal_options\x18\x05 \x01(\x0b\x32\x15.gitaly.GlobalOptions\">\n\x19LastCommitForPathResponse\x12!\n\x06\x63ommit\x18\x01 \x01(\x0b\x32\x11.gitaly.GitCommit\"\xd9\x01\n\x1dListLastCommitsForTreeRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\t\x12\x0c\n\x04path\x18\x03 \x01(\x0c\x12\r\n\x05limit\x18\x04 \x01(\x05\x12\x0e\n\x06offset\x18\x05 \x01(\x05\x12\x1c\n\x10literal_pathspec\x18\x06 \x01(\x08\x42\x02\x18\x01\x12-\n\x0eglobal_options\x18\x07 \x01(\x0b\x32\x15.gitaly.GlobalOptions\"\xbb\x01\n\x1eListLastCommitsForTreeResponse\x12\x45\n\x07\x63ommits\x18\x01 \x03(\x0b\x32\x34.gitaly.ListLastCommitsForTreeResponse.CommitForTree\x1aR\n\rCommitForTree\x12!\n\x06\x63ommit\x18\x02 \x01(\x0b\x32\x11.gitaly.GitCommit\x12\x12\n\npath_bytes\x18\x04 \x01(\x0cJ\x04\x08\x01\x10\x02J\x04\x08\x03\x10\x04\"\xc4\x01\n\x17\x43ommitsByMessageRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\x12\x0e\n\x06offset\x18\x03 \x01(\x05\x12\r\n\x05limit\x18\x04 \x01(\x05\x12\x0c\n\x04path\x18\x05 \x01(\x0c\x12\r\n\x05query\x18\x06 \x01(\t\x12-\n\x0eglobal_options\x18\x07 \x01(\x0b\x32\x15.gitaly.GlobalOptions\">\n\x18\x43ommitsByMessageResponse\x12\"\n\x07\x63ommits\x18\x01 \x03(\x0b\x32\x11.gitaly.GitCommit\"]\n\x1f\x46ilterShasWithSignaturesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0c\n\x04shas\x18\x02 \x03(\x0c\"0\n FilterShasWithSignaturesResponse\x12\x0c\n\x04shas\x18\x01 \x03(\x0c\"^\n\x1aGetCommitSignaturesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x12\n\ncommit_ids\x18\x02 \x03(\t\"\xda\x01\n\x1bGetCommitSignaturesResponse\x12\x11\n\tcommit_id\x18\x01 \x01(\t\x12\x11\n\tsignature\x18\x02 \x01(\x0c\x12\x13\n\x0bsigned_text\x18\x03 \x01(\x0c\x12:\n\x06signer\x18\x04 \x01(\x0e\x32*.gitaly.GetCommitSignaturesResponse.Signer\"D\n\x06Signer\x12\x16\n\x12SIGNER_UNSPECIFIED\x10\x00\x12\x0f\n\x0bSIGNER_USER\x10\x01\x12\x11\n\rSIGNER_SYSTEM\x10\x02\"\\\n\x18GetCommitMessagesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x12\n\ncommit_ids\x18\x02 \x03(\t\"?\n\x19GetCommitMessagesResponse\x12\x11\n\tcommit_id\x18\x01 \x01(\t\x12\x0f\n\x07message\x18\x02 \x01(\x0c\"[\n\x18\x43heckObjectsExistRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\trevisions\x18\x02 \x03(\x0c\"\x96\x01\n\x19\x43heckObjectsExistResponse\x12\x46\n\trevisions\x18\x01 \x03(\x0b\x32\x33.gitaly.CheckObjectsExistResponse.RevisionExistence\x1a\x31\n\x11RevisionExistence\x12\x0c\n\x04name\x18\x01 \x01(\x0c\x12\x0e\n\x06\x65xists\x18\x02 \x01(\x08\x32\xf1\x10\n\rCommitService\x12P\n\x0bListCommits\x12\x1a.gitaly.ListCommitsRequest\x1a\x1b.gitaly.ListCommitsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12Y\n\x0eListAllCommits\x12\x1d.gitaly.ListAllCommitsRequest\x1a\x1e.gitaly.ListAllCommitsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12]\n\x10\x43ommitIsAncestor\x12\x1f.gitaly.CommitIsAncestorRequest\x1a .gitaly.CommitIsAncestorResponse\"\x06\xfa\x97(\x02\x08\x02\x12J\n\tTreeEntry\x12\x18.gitaly.TreeEntryRequest\x1a\x19.gitaly.TreeEntryResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12Q\n\x0c\x43ountCommits\x12\x1b.gitaly.CountCommitsRequest\x1a\x1c.gitaly.CountCommitsResponse\"\x06\xfa\x97(\x02\x08\x02\x12l\n\x15\x43ountDivergingCommits\x12$.gitaly.CountDivergingCommitsRequest\x1a%.gitaly.CountDivergingCommitsResponse\"\x06\xfa\x97(\x02\x08\x02\x12Y\n\x0eGetTreeEntries\x12\x1d.gitaly.GetTreeEntriesRequest\x1a\x1e.gitaly.GetTreeEntriesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12J\n\tListFiles\x12\x18.gitaly.ListFilesRequest\x1a\x19.gitaly.ListFilesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12K\n\nFindCommit\x12\x19.gitaly.FindCommitRequest\x1a\x1a.gitaly.FindCommitResponse\"\x06\xfa\x97(\x02\x08\x02\x12N\n\x0b\x43ommitStats\x12\x1a.gitaly.CommitStatsRequest\x1a\x1b.gitaly.CommitStatsResponse\"\x06\xfa\x97(\x02\x08\x02\x12Y\n\x0e\x46indAllCommits\x12\x1d.gitaly.FindAllCommitsRequest\x1a\x1e.gitaly.FindAllCommitsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12P\n\x0b\x46indCommits\x12\x1a.gitaly.FindCommitsRequest\x1a\x1b.gitaly.FindCommitsResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12Z\n\x0f\x43ommitLanguages\x12\x1e.gitaly.CommitLanguagesRequest\x1a\x1f.gitaly.CommitLanguagesResponse\"\x06\xfa\x97(\x02\x08\x02\x12G\n\x08RawBlame\x12\x17.gitaly.RawBlameRequest\x1a\x18.gitaly.RawBlameResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12`\n\x11LastCommitForPath\x12 .gitaly.LastCommitForPathRequest\x1a!.gitaly.LastCommitForPathResponse\"\x06\xfa\x97(\x02\x08\x02\x12q\n\x16ListLastCommitsForTree\x12%.gitaly.ListLastCommitsForTreeRequest\x1a&.gitaly.ListLastCommitsForTreeResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12_\n\x10\x43ommitsByMessage\x12\x1f.gitaly.CommitsByMessageRequest\x1a .gitaly.CommitsByMessageResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12_\n\x10ListCommitsByOid\x12\x1f.gitaly.ListCommitsByOidRequest\x1a .gitaly.ListCommitsByOidResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12k\n\x14ListCommitsByRefName\x12#.gitaly.ListCommitsByRefNameRequest\x1a$.gitaly.ListCommitsByRefNameResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12y\n\x18\x46ilterShasWithSignatures\x12\'.gitaly.FilterShasWithSignaturesRequest\x1a(.gitaly.FilterShasWithSignaturesResponse\"\x06\xfa\x97(\x02\x08\x02(\x01\x30\x01\x12h\n\x13GetCommitSignatures\x12\".gitaly.GetCommitSignaturesRequest\x1a#.gitaly.GetCommitSignaturesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\x62\n\x11GetCommitMessages\x12 .gitaly.GetCommitMessagesRequest\x1a!.gitaly.GetCommitMessagesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\x64\n\x11\x43heckObjectsExist\x12 .gitaly.CheckObjectsExistRequest\x1a!.gitaly.CheckObjectsExistResponse\"\x06\xfa\x97(\x02\x08\x02(\x01\x30\x01\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'commit_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _LISTCOMMITSREQUEST.fields_by_name['repository']._options = None
  _LISTCOMMITSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTALLCOMMITSREQUEST.fields_by_name['repository']._options = None
  _LISTALLCOMMITSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _COMMITSTATSREQUEST.fields_by_name['repository']._options = None
  _COMMITSTATSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _COMMITISANCESTORREQUEST.fields_by_name['repository']._options = None
  _COMMITISANCESTORREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _TREEENTRYREQUEST.fields_by_name['repository']._options = None
  _TREEENTRYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _COUNTCOMMITSREQUEST.fields_by_name['repository']._options = None
  _COUNTCOMMITSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _COUNTDIVERGINGCOMMITSREQUEST.fields_by_name['repository']._options = None
  _COUNTDIVERGINGCOMMITSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETTREEENTRIESREQUEST.fields_by_name['repository']._options = None
  _GETTREEENTRIESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTFILESREQUEST.fields_by_name['repository']._options = None
  _LISTFILESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDCOMMITREQUEST.fields_by_name['repository']._options = None
  _FINDCOMMITREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTCOMMITSBYOIDREQUEST.fields_by_name['repository']._options = None
  _LISTCOMMITSBYOIDREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTCOMMITSBYREFNAMEREQUEST.fields_by_name['repository']._options = None
  _LISTCOMMITSBYREFNAMEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDALLCOMMITSREQUEST.fields_by_name['repository']._options = None
  _FINDALLCOMMITSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDCOMMITSREQUEST.fields_by_name['repository']._options = None
  _FINDCOMMITSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDCOMMITSREQUEST.fields_by_name['disable_walk']._options = None
  _FINDCOMMITSREQUEST.fields_by_name['disable_walk']._serialized_options = b'\030\001'
  _COMMITLANGUAGESREQUEST.fields_by_name['repository']._options = None
  _COMMITLANGUAGESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _RAWBLAMEREQUEST.fields_by_name['repository']._options = None
  _RAWBLAMEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LASTCOMMITFORPATHREQUEST.fields_by_name['repository']._options = None
  _LASTCOMMITFORPATHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTLASTCOMMITSFORTREEREQUEST.fields_by_name['repository']._options = None
  _LISTLASTCOMMITSFORTREEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _LISTLASTCOMMITSFORTREEREQUEST.fields_by_name['literal_pathspec']._options = None
  _LISTLASTCOMMITSFORTREEREQUEST.fields_by_name['literal_pathspec']._serialized_options = b'\030\001'
  _COMMITSBYMESSAGEREQUEST.fields_by_name['repository']._options = None
  _COMMITSBYMESSAGEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FILTERSHASWITHSIGNATURESREQUEST.fields_by_name['repository']._options = None
  _FILTERSHASWITHSIGNATURESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETCOMMITSIGNATURESREQUEST.fields_by_name['repository']._options = None
  _GETCOMMITSIGNATURESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETCOMMITMESSAGESREQUEST.fields_by_name['repository']._options = None
  _GETCOMMITMESSAGESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CHECKOBJECTSEXISTREQUEST.fields_by_name['repository']._options = None
  _CHECKOBJECTSEXISTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _COMMITSERVICE.methods_by_name['ListCommits']._options = None
  _COMMITSERVICE.methods_by_name['ListCommits']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['ListAllCommits']._options = None
  _COMMITSERVICE.methods_by_name['ListAllCommits']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['CommitIsAncestor']._options = None
  _COMMITSERVICE.methods_by_name['CommitIsAncestor']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['TreeEntry']._options = None
  _COMMITSERVICE.methods_by_name['TreeEntry']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['CountCommits']._options = None
  _COMMITSERVICE.methods_by_name['CountCommits']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['CountDivergingCommits']._options = None
  _COMMITSERVICE.methods_by_name['CountDivergingCommits']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['GetTreeEntries']._options = None
  _COMMITSERVICE.methods_by_name['GetTreeEntries']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['ListFiles']._options = None
  _COMMITSERVICE.methods_by_name['ListFiles']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['FindCommit']._options = None
  _COMMITSERVICE.methods_by_name['FindCommit']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['CommitStats']._options = None
  _COMMITSERVICE.methods_by_name['CommitStats']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['FindAllCommits']._options = None
  _COMMITSERVICE.methods_by_name['FindAllCommits']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['FindCommits']._options = None
  _COMMITSERVICE.methods_by_name['FindCommits']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['CommitLanguages']._options = None
  _COMMITSERVICE.methods_by_name['CommitLanguages']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['RawBlame']._options = None
  _COMMITSERVICE.methods_by_name['RawBlame']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['LastCommitForPath']._options = None
  _COMMITSERVICE.methods_by_name['LastCommitForPath']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['ListLastCommitsForTree']._options = None
  _COMMITSERVICE.methods_by_name['ListLastCommitsForTree']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['CommitsByMessage']._options = None
  _COMMITSERVICE.methods_by_name['CommitsByMessage']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['ListCommitsByOid']._options = None
  _COMMITSERVICE.methods_by_name['ListCommitsByOid']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['ListCommitsByRefName']._options = None
  _COMMITSERVICE.methods_by_name['ListCommitsByRefName']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['FilterShasWithSignatures']._options = None
  _COMMITSERVICE.methods_by_name['FilterShasWithSignatures']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['GetCommitSignatures']._options = None
  _COMMITSERVICE.methods_by_name['GetCommitSignatures']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['GetCommitMessages']._options = None
  _COMMITSERVICE.methods_by_name['GetCommitMessages']._serialized_options = b'\372\227(\002\010\002'
  _COMMITSERVICE.methods_by_name['CheckObjectsExist']._options = None
  _COMMITSERVICE.methods_by_name['CheckObjectsExist']._serialized_options = b'\372\227(\002\010\002'
  _globals['_LISTCOMMITSREQUEST']._serialized_start=98
  _globals['_LISTCOMMITSREQUEST']._serialized_end=566
  _globals['_LISTCOMMITSREQUEST_ORDER']._serialized_start=529
  _globals['_LISTCOMMITSREQUEST_ORDER']._serialized_end=566
  _globals['_LISTCOMMITSRESPONSE']._serialized_start=568
  _globals['_LISTCOMMITSRESPONSE']._serialized_end=625
  _globals['_LISTALLCOMMITSREQUEST']._serialized_start=627
  _globals['_LISTALLCOMMITSREQUEST']._serialized_end=752
  _globals['_LISTALLCOMMITSRESPONSE']._serialized_start=754
  _globals['_LISTALLCOMMITSRESPONSE']._serialized_end=814
  _globals['_COMMITSTATSREQUEST']._serialized_start=816
  _globals['_COMMITSTATSREQUEST']._serialized_end=900
  _globals['_COMMITSTATSRESPONSE']._serialized_start=902
  _globals['_COMMITSTATSRESPONSE']._serialized_end=974
  _globals['_COMMITISANCESTORREQUEST']._serialized_start=976
  _globals['_COMMITISANCESTORREQUEST']._serialized_end=1086
  _globals['_COMMITISANCESTORRESPONSE']._serialized_start=1088
  _globals['_COMMITISANCESTORRESPONSE']._serialized_end=1129
  _globals['_TREEENTRYREQUEST']._serialized_start=1132
  _globals['_TREEENTRYREQUEST']._serialized_end=1261
  _globals['_TREEENTRYRESPONSE']._serialized_start=1264
  _globals['_TREEENTRYRESPONSE']._serialized_end=1445
  _globals['_TREEENTRYRESPONSE_OBJECTTYPE']._serialized_start=1392
  _globals['_TREEENTRYRESPONSE_OBJECTTYPE']._serialized_end=1445
  _globals['_COUNTCOMMITSREQUEST']._serialized_start=1448
  _globals['_COUNTCOMMITSREQUEST']._serialized_end=1735
  _globals['_COUNTCOMMITSRESPONSE']._serialized_start=1737
  _globals['_COUNTCOMMITSRESPONSE']._serialized_end=1774
  _globals['_COUNTDIVERGINGCOMMITSREQUEST']._serialized_start=1777
  _globals['_COUNTDIVERGINGCOMMITSREQUEST']._serialized_end=1916
  _globals['_COUNTDIVERGINGCOMMITSRESPONSE']._serialized_start=1918
  _globals['_COUNTDIVERGINGCOMMITSRESPONSE']._serialized_end=1990
  _globals['_TREEENTRY']._serialized_start=1993
  _globals['_TREEENTRY']._serialized_end=2188
  _globals['_TREEENTRY_ENTRYTYPE']._serialized_start=2129
  _globals['_TREEENTRY_ENTRYTYPE']._serialized_end=2172
  _globals['_GETTREEENTRIESREQUEST']._serialized_start=2191
  _globals['_GETTREEENTRIESREQUEST']._serialized_end=2484
  _globals['_GETTREEENTRIESREQUEST_SORTBY']._serialized_start=2446
  _globals['_GETTREEENTRIESREQUEST_SORTBY']._serialized_end=2484
  _globals['_GETTREEENTRIESRESPONSE']._serialized_start=2486
  _globals['_GETTREEENTRIESRESPONSE']._serialized_end=2599
  _globals['_GETTREEENTRIESERROR']._serialized_start=2601
  _globals['_GETTREEENTRIESERROR']._serialized_end=2720
  _globals['_LISTFILESREQUEST']._serialized_start=2722
  _globals['_LISTFILESREQUEST']._serialized_end=2804
  _globals['_LISTFILESRESPONSE']._serialized_start=2806
  _globals['_LISTFILESRESPONSE']._serialized_end=2840
  _globals['_FINDCOMMITREQUEST']._serialized_start=2842
  _globals['_FINDCOMMITREQUEST']._serialized_end=2943
  _globals['_FINDCOMMITRESPONSE']._serialized_start=2945
  _globals['_FINDCOMMITRESPONSE']._serialized_end=3000
  _globals['_LISTCOMMITSBYOIDREQUEST']._serialized_start=3002
  _globals['_LISTCOMMITSBYOIDREQUEST']._serialized_end=3086
  _globals['_LISTCOMMITSBYOIDRESPONSE']._serialized_start=3088
  _globals['_LISTCOMMITSBYOIDRESPONSE']._serialized_end=3150
  _globals['_LISTCOMMITSBYREFNAMEREQUEST']._serialized_start=3152
  _globals['_LISTCOMMITSBYREFNAMEREQUEST']._serialized_end=3246
  _globals['_LISTCOMMITSBYREFNAMERESPONSE']._serialized_start=3249
  _globals['_LISTCOMMITSBYREFNAMERESPONSE']._serialized_end=3426
  _globals['_LISTCOMMITSBYREFNAMERESPONSE_COMMITFORREF']._serialized_start=3353
  _globals['_LISTCOMMITSBYREFNAMERESPONSE_COMMITFORREF']._serialized_end=3420
  _globals['_FINDALLCOMMITSREQUEST']._serialized_start=3429
  _globals['_FINDALLCOMMITSREQUEST']._serialized_end=3640
  _globals['_FINDALLCOMMITSREQUEST_ORDER']._serialized_start=529
  _globals['_FINDALLCOMMITSREQUEST_ORDER']._serialized_end=566
  _globals['_FINDALLCOMMITSRESPONSE']._serialized_start=3642
  _globals['_FINDALLCOMMITSRESPONSE']._serialized_end=3702
  _globals['_FINDCOMMITSREQUEST']._serialized_start=3705
  _globals['_FINDCOMMITSREQUEST']._serialized_end=4237
  _globals['_FINDCOMMITSREQUEST_ORDER']._serialized_start=529
  _globals['_FINDCOMMITSREQUEST_ORDER']._serialized_end=556
  _globals['_FINDCOMMITSRESPONSE']._serialized_start=4239
  _globals['_FINDCOMMITSRESPONSE']._serialized_end=4296
  _globals['_FINDCOMMITSERROR']._serialized_start=4299
  _globals['_FINDCOMMITSERROR']._serialized_end=4485
  _globals['_COMMITLANGUAGESREQUEST']._serialized_start=4487
  _globals['_COMMITLANGUAGESREQUEST']._serialized_end=4575
  _globals['_COMMITLANGUAGESRESPONSE']._serialized_start=4578
  _globals['_COMMITLANGUAGESRESPONSE']._serialized_end=4753
  _globals['_COMMITLANGUAGESRESPONSE_LANGUAGE']._serialized_start=4666
  _globals['_COMMITLANGUAGESRESPONSE_LANGUAGE']._serialized_end=4753
  _globals['_RAWBLAMEREQUEST']._serialized_start=4755
  _globals['_RAWBLAMEREQUEST']._serialized_end=4865
  _globals['_RAWBLAMERESPONSE']._serialized_start=4867
  _globals['_RAWBLAMERESPONSE']._serialized_end=4899
  _globals['_RAWBLAMEERROR']._serialized_start=4902
  _globals['_RAWBLAMEERROR']._serialized_end=5083
  _globals['_RAWBLAMEERROR_OUTOFRANGEERROR']._serialized_start=5035
  _globals['_RAWBLAMEERROR_OUTOFRANGEERROR']._serialized_end=5074
  _globals['_LASTCOMMITFORPATHREQUEST']._serialized_start=5086
  _globals['_LASTCOMMITFORPATHREQUEST']._serialized_end=5263
  _globals['_LASTCOMMITFORPATHRESPONSE']._serialized_start=5265
  _globals['_LASTCOMMITFORPATHRESPONSE']._serialized_end=5327
  _globals['_LISTLASTCOMMITSFORTREEREQUEST']._serialized_start=5330
  _globals['_LISTLASTCOMMITSFORTREEREQUEST']._serialized_end=5547
  _globals['_LISTLASTCOMMITSFORTREERESPONSE']._serialized_start=5550
  _globals['_LISTLASTCOMMITSFORTREERESPONSE']._serialized_end=5737
  _globals['_LISTLASTCOMMITSFORTREERESPONSE_COMMITFORTREE']._serialized_start=5655
  _globals['_LISTLASTCOMMITSFORTREERESPONSE_COMMITFORTREE']._serialized_end=5737
  _globals['_COMMITSBYMESSAGEREQUEST']._serialized_start=5740
  _globals['_COMMITSBYMESSAGEREQUEST']._serialized_end=5936
  _globals['_COMMITSBYMESSAGERESPONSE']._serialized_start=5938
  _globals['_COMMITSBYMESSAGERESPONSE']._serialized_end=6000
  _globals['_FILTERSHASWITHSIGNATURESREQUEST']._serialized_start=6002
  _globals['_FILTERSHASWITHSIGNATURESREQUEST']._serialized_end=6095
  _globals['_FILTERSHASWITHSIGNATURESRESPONSE']._serialized_start=6097
  _globals['_FILTERSHASWITHSIGNATURESRESPONSE']._serialized_end=6145
  _globals['_GETCOMMITSIGNATURESREQUEST']._serialized_start=6147
  _globals['_GETCOMMITSIGNATURESREQUEST']._serialized_end=6241
  _globals['_GETCOMMITSIGNATURESRESPONSE']._serialized_start=6244
  _globals['_GETCOMMITSIGNATURESRESPONSE']._serialized_end=6462
  _globals['_GETCOMMITSIGNATURESRESPONSE_SIGNER']._serialized_start=6394
  _globals['_GETCOMMITSIGNATURESRESPONSE_SIGNER']._serialized_end=6462
  _globals['_GETCOMMITMESSAGESREQUEST']._serialized_start=6464
  _globals['_GETCOMMITMESSAGESREQUEST']._serialized_end=6556
  _globals['_GETCOMMITMESSAGESRESPONSE']._serialized_start=6558
  _globals['_GETCOMMITMESSAGESRESPONSE']._serialized_end=6621
  _globals['_CHECKOBJECTSEXISTREQUEST']._serialized_start=6623
  _globals['_CHECKOBJECTSEXISTREQUEST']._serialized_end=6714
  _globals['_CHECKOBJECTSEXISTRESPONSE']._serialized_start=6717
  _globals['_CHECKOBJECTSEXISTRESPONSE']._serialized_end=6867
  _globals['_CHECKOBJECTSEXISTRESPONSE_REVISIONEXISTENCE']._serialized_start=6818
  _globals['_CHECKOBJECTSEXISTRESPONSE_REVISIONEXISTENCE']._serialized_end=6867
  _globals['_COMMITSERVICE']._serialized_start=6870
  _globals['_COMMITSERVICE']._serialized_end=9031
# @@protoc_insertion_point(module_scope)
