# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: repository.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10repository.proto\x12\x06gitaly\x1a\nlint.proto\x1a\x0cshared.proto\"G\n\x17RepositoryExistsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"*\n\x18RepositoryExistsResponse\x12\x0e\n\x06\x65xists\x18\x01 \x01(\x08\"E\n\x15RepositorySizeRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"&\n\x16RepositorySizeResponse\x12\x0c\n\x04size\x18\x01 \x01(\x03\"E\n\x15RepositoryInfoRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\xbb\x02\n\x16RepositoryInfoResponse\x12\x0c\n\x04size\x18\x01 \x01(\x04\x12\x41\n\nreferences\x18\x02 \x01(\x0b\x32-.gitaly.RepositoryInfoResponse.ReferencesInfo\x12;\n\x07objects\x18\x03 \x01(\x0b\x32*.gitaly.RepositoryInfoResponse.ObjectsInfo\x1a:\n\x0eReferencesInfo\x12\x13\n\x0bloose_count\x18\x01 \x01(\x04\x12\x13\n\x0bpacked_size\x18\x02 \x01(\x04\x1aW\n\x0bObjectsInfo\x12\x0c\n\x04size\x18\x01 \x01(\x04\x12\x13\n\x0brecent_size\x18\x02 \x01(\x04\x12\x12\n\nstale_size\x18\x03 \x01(\x04\x12\x11\n\tkeep_size\x18\x04 \x01(\x04\"U\n\x12ObjectsSizeRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\trevisions\x18\x02 \x03(\x0c\"#\n\x13ObjectsSizeResponse\x12\x0c\n\x04size\x18\x01 \x01(\x04\"C\n\x13ObjectFormatRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"<\n\x14ObjectFormatResponse\x12$\n\x06\x66ormat\x18\x01 \x01(\x0e\x32\x14.gitaly.ObjectFormat\"[\n\x19\x41pplyGitattributesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\"\x1c\n\x1a\x41pplyGitattributesResponse\"e\n\x12\x46\x65tchBundleRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x13\n\x0bupdate_head\x18\x03 \x01(\x08\"\x15\n\x13\x46\x65tchBundleResponse\"\x82\x02\n\x12\x46\x65tchRemoteRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\r\n\x05\x66orce\x18\x03 \x01(\x08\x12\x0f\n\x07no_tags\x18\x04 \x01(\x08\x12\x0f\n\x07timeout\x18\x05 \x01(\x05\x12\x0f\n\x07ssh_key\x18\x06 \x01(\t\x12\x13\n\x0bknown_hosts\x18\x07 \x01(\t\x12\x10\n\x08no_prune\x18\t \x01(\x08\x12%\n\rremote_params\x18\n \x01(\x0b\x32\x0e.gitaly.Remote\x12\x1a\n\x12\x63heck_tags_changed\x18\x0b \x01(\x08J\x04\x08\x08\x10\tJ\x04\x08\x02\x10\x03R\x06remote\"+\n\x13\x46\x65tchRemoteResponse\x12\x14\n\x0ctags_changed\x18\x01 \x01(\x08\"\x8c\x01\n\x17\x43reateRepositoryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x16\n\x0e\x64\x65\x66\x61ult_branch\x18\x02 \x01(\x0c\x12+\n\robject_format\x18\x03 \x01(\x0e\x32\x14.gitaly.ObjectFormat\"\x1a\n\x18\x43reateRepositoryResponse\"\x99\x02\n\x11GetArchiveRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\tcommit_id\x18\x02 \x01(\t\x12\x0e\n\x06prefix\x18\x03 \x01(\t\x12\x30\n\x06\x66ormat\x18\x04 \x01(\x0e\x32 .gitaly.GetArchiveRequest.Format\x12\x0c\n\x04path\x18\x05 \x01(\x0c\x12\x0f\n\x07\x65xclude\x18\x06 \x03(\x0c\x12\x12\n\nelide_path\x18\x07 \x01(\x08\x12\x19\n\x11include_lfs_blobs\x18\x08 \x01(\x08\"3\n\x06\x46ormat\x12\x07\n\x03ZIP\x10\x00\x12\x07\n\x03TAR\x10\x01\x12\n\n\x06TAR_GZ\x10\x02\x12\x0b\n\x07TAR_BZ2\x10\x03\"\"\n\x12GetArchiveResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"G\n\x17HasLocalBranchesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\")\n\x18HasLocalBranchesResponse\x12\r\n\x05value\x18\x01 \x01(\x08\"\xc3\x01\n\x18\x46\x65tchSourceBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12-\n\x11source_repository\x18\x02 \x01(\x0b\x32\x12.gitaly.Repository\x12\x15\n\rsource_branch\x18\x03 \x01(\x0c\x12\x12\n\ntarget_ref\x18\x04 \x01(\x0c\x12\x1f\n\x17\x65xpected_target_old_oid\x18\x05 \x01(\t\"+\n\x19\x46\x65tchSourceBranchResponse\x12\x0e\n\x06result\x18\x01 \x01(\x08\";\n\x0b\x46sckRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\x1d\n\x0c\x46sckResponse\x12\r\n\x05\x65rror\x18\x01 \x01(\x0c\"\x87\x01\n\x0fWriteRefRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0b\n\x03ref\x18\x02 \x01(\x0c\x12\x10\n\x08revision\x18\x03 \x01(\x0c\x12\x14\n\x0cold_revision\x18\x04 \x01(\x0cJ\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07R\x05\x66orce\"\x18\n\x10WriteRefResponseJ\x04\x08\x01\x10\x02\"W\n\x14\x46indMergeBaseRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\trevisions\x18\x02 \x03(\x0c\"%\n\x15\x46indMergeBaseResponse\x12\x0c\n\x04\x62\x61se\x18\x01 \x01(\t\"\x82\x01\n\x11\x43reateForkRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12-\n\x11source_repository\x18\x02 \x01(\x0b\x32\x12.gitaly.Repository\x12\x10\n\x08revision\x18\x03 \x01(\x0c\"\x14\n\x12\x43reateForkResponse\"\xb9\x01\n\x1e\x43reateRepositoryFromURLRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0b\n\x03url\x18\x02 \x01(\t\x12!\n\x19http_authorization_header\x18\x04 \x01(\t\x12\x0e\n\x06mirror\x18\x05 \x01(\x08\x12\x18\n\x10resolved_address\x18\x06 \x01(\tJ\x04\x08\x03\x10\x04R\thttp_host\"!\n\x1f\x43reateRepositoryFromURLResponse\"C\n\x13\x43reateBundleRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"$\n\x14\x43reateBundleResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"`\n\x1e\x43reateBundleFromRefListRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08patterns\x18\x02 \x03(\x0c\"/\n\x1f\x43reateBundleFromRefListResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"H\n\x18GenerateBundleURIRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\x1b\n\x19GenerateBundleURIResponse\"@\n\x10GetConfigRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"!\n\x11GetConfigResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"W\n\x19RestoreCustomHooksRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"S\n\x15SetCustomHooksRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"\x1c\n\x1aRestoreCustomHooksResponse\"\x18\n\x16SetCustomHooksResponse\"H\n\x18\x42\x61\x63kupCustomHooksRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"E\n\x15GetCustomHooksRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\")\n\x19\x42\x61\x63kupCustomHooksResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"&\n\x16GetCustomHooksResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"_\n!CreateRepositoryFromBundleRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"$\n\"CreateRepositoryFromBundleResponse\"B\n\x12\x46indLicenseRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\x8c\x01\n\x13\x46indLicenseResponse\x12\x1a\n\x12license_short_name\x18\x01 \x01(\t\x12\x14\n\x0clicense_name\x18\x02 \x01(\t\x12\x13\n\x0blicense_url\x18\x03 \x01(\t\x12\x14\n\x0clicense_path\x18\x04 \x01(\t\x12\x18\n\x10license_nickname\x18\x05 \x01(\t\"H\n\x18GetInfoAttributesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"/\n\x19GetInfoAttributesResponse\x12\x12\n\nattributes\x18\x01 \x01(\x0c\"H\n\x18\x43\x61lculateChecksumRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"-\n\x19\x43\x61lculateChecksumResponse\x12\x10\n\x08\x63hecksum\x18\x01 \x01(\t\"B\n\x12GetSnapshotRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"#\n\x13GetSnapshotResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\xa3\x01\n#CreateRepositoryFromSnapshotRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08http_url\x18\x02 \x01(\t\x12\x11\n\thttp_auth\x18\x03 \x01(\t\x12\x18\n\x10resolved_address\x18\x05 \x01(\tJ\x04\x08\x04\x10\x05R\thttp_host\"&\n$CreateRepositoryFromSnapshotResponse\"p\n\x14GetRawChangesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x15\n\rfrom_revision\x18\x02 \x01(\t\x12\x13\n\x0bto_revision\x18\x03 \x01(\t\"\xbc\x03\n\x15GetRawChangesResponse\x12<\n\x0braw_changes\x18\x01 \x03(\x0b\x32\'.gitaly.GetRawChangesResponse.RawChange\x1a\xe4\x02\n\tRawChange\x12\x0f\n\x07\x62lob_id\x18\x01 \x01(\t\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x44\n\toperation\x18\x05 \x01(\x0e\x32\x31.gitaly.GetRawChangesResponse.RawChange.Operation\x12\x10\n\x08old_mode\x18\x07 \x01(\x05\x12\x10\n\x08new_mode\x18\x08 \x01(\x05\x12\x16\n\x0enew_path_bytes\x18\t \x01(\x0c\x12\x16\n\x0eold_path_bytes\x18\n \x01(\x0c\"i\n\tOperation\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05\x41\x44\x44\x45\x44\x10\x01\x12\n\n\x06\x43OPIED\x10\x02\x12\x0b\n\x07\x44\x45LETED\x10\x03\x12\x0c\n\x08MODIFIED\x10\x04\x12\x0b\n\x07RENAMED\x10\x05\x12\x10\n\x0cTYPE_CHANGED\x10\x06J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x06\x10\x07R\x08new_pathR\x08old_pathR\rraw_operation\"\x93\x01\n\x18SearchFilesByNameRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\r\n\x05query\x18\x02 \x01(\t\x12\x0b\n\x03ref\x18\x03 \x01(\x0c\x12\x0e\n\x06\x66ilter\x18\x04 \x01(\t\x12\r\n\x05limit\x18\x05 \x01(\r\x12\x0e\n\x06offset\x18\x06 \x01(\r\"*\n\x19SearchFilesByNameResponse\x12\r\n\x05\x66iles\x18\x01 \x03(\x0c\"\x7f\n\x1bSearchFilesByContentRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\r\n\x05query\x18\x02 \x01(\t\x12\x0b\n\x03ref\x18\x03 \x01(\x0cJ\x04\x08\x04\x10\x05R\x10\x63hunked_response\"W\n\x1cSearchFilesByContentResponse\x12\x12\n\nmatch_data\x18\x02 \x01(\x0c\x12\x14\n\x0c\x65nd_of_match\x18\x03 \x01(\x08J\x04\x08\x01\x10\x02R\x07matches\"\x87\x01\n\x06Remote\x12\x0b\n\x03url\x18\x01 \x01(\t\x12!\n\x19http_authorization_header\x18\x03 \x01(\t\x12\x16\n\x0emirror_refmaps\x18\x04 \x03(\t\x12\x18\n\x10resolved_address\x18\x06 \x01(\tJ\x04\x08\x02\x10\x03J\x04\x08\x05\x10\x06R\x04nameR\thttp_host\"M\n\x1dGetObjectDirectorySizeRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\".\n\x1eGetObjectDirectorySizeResponse\x12\x0c\n\x04size\x18\x01 \x01(\x03\"G\n\x17RemoveRepositoryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\x1a\n\x18RemoveRepositoryResponse\"n\n\x1aReplicateRepositoryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\"\n\x06source\x18\x02 \x01(\x0b\x32\x12.gitaly.Repository\"\x1d\n\x1bReplicateRepositoryResponse\"\xdb\x01\n\x19OptimizeRepositoryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12<\n\x08strategy\x18\x02 \x01(\x0e\x32*.gitaly.OptimizeRepositoryRequest.Strategy\"R\n\x08Strategy\x12\x18\n\x14STRATEGY_UNSPECIFIED\x10\x00\x12\x18\n\x14STRATEGY_HEURISTICAL\x10\x01\x12\x12\n\x0eSTRATEGY_EAGER\x10\x02\"\x1c\n\x1aOptimizeRepositoryResponse\"N\n\x1ePruneUnreachableObjectsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"!\n\x1fPruneUnreachableObjectsResponse\"\x9e\x01\n\x17\x42\x61\x63kupRepositoryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12-\n\x11vanity_repository\x18\x02 \x01(\x0b\x32\x12.gitaly.Repository\x12\x11\n\tbackup_id\x18\x03 \x01(\t\x12\x13\n\x0bincremental\x18\x04 \x01(\x08\"*\n\x18\x42\x61\x63kupRepositoryResponse\x1a\x0e\n\x0cSkippedError\"\xa1\x01\n\x18RestoreRepositoryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12-\n\x11vanity_repository\x18\x02 \x01(\x0b\x32\x12.gitaly.Repository\x12\x11\n\tbackup_id\x18\x03 \x01(\t\x12\x15\n\ralways_create\x18\x04 \x01(\x08\"+\n\x19RestoreRepositoryResponse\x1a\x0e\n\x0cSkippedError\"}\n\x18GetFileAttributesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\x12\x12\n\nattributes\x18\x03 \x03(\t\x12\r\n\x05paths\x18\x04 \x03(\t\"\xa6\x01\n\x19GetFileAttributesResponse\x12H\n\x0f\x61ttribute_infos\x18\x01 \x03(\x0b\x32/.gitaly.GetFileAttributesResponse.AttributeInfo\x1a?\n\rAttributeInfo\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x11\n\tattribute\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\"A\n\x11\x46\x61stExportRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\"\n\x12\x46\x61stExportResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x32\xf7\x1f\n\x11RepositoryService\x12]\n\x10RepositoryExists\x12\x1f.gitaly.RepositoryExistsRequest\x1a .gitaly.RepositoryExistsResponse\"\x06\xfa\x97(\x02\x08\x02\x12W\n\x0eRepositorySize\x12\x1d.gitaly.RepositorySizeRequest\x1a\x1e.gitaly.RepositorySizeResponse\"\x06\xfa\x97(\x02\x08\x02\x12W\n\x0eRepositoryInfo\x12\x1d.gitaly.RepositoryInfoRequest\x1a\x1e.gitaly.RepositoryInfoResponse\"\x06\xfa\x97(\x02\x08\x02\x12P\n\x0bObjectsSize\x12\x1a.gitaly.ObjectsSizeRequest\x1a\x1b.gitaly.ObjectsSizeResponse\"\x06\xfa\x97(\x02\x08\x02(\x01\x12Q\n\x0cObjectFormat\x12\x1b.gitaly.ObjectFormatRequest\x1a\x1c.gitaly.ObjectFormatResponse\"\x06\xfa\x97(\x02\x08\x02\x12\x66\n\x12\x41pplyGitattributes\x12!.gitaly.ApplyGitattributesRequest\x1a\".gitaly.ApplyGitattributesResponse\"\t\x88\x02\x01\xfa\x97(\x02\x08\x01\x12N\n\x0b\x46\x65tchRemote\x12\x1a.gitaly.FetchRemoteRequest\x1a\x1b.gitaly.FetchRemoteResponse\"\x06\xfa\x97(\x02\x08\x01\x12]\n\x10\x43reateRepository\x12\x1f.gitaly.CreateRepositoryRequest\x1a .gitaly.CreateRepositoryResponse\"\x06\xfa\x97(\x02\x08\x01\x12M\n\nGetArchive\x12\x19.gitaly.GetArchiveRequest\x1a\x1a.gitaly.GetArchiveResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12]\n\x10HasLocalBranches\x12\x1f.gitaly.HasLocalBranchesRequest\x1a .gitaly.HasLocalBranchesResponse\"\x06\xfa\x97(\x02\x08\x02\x12`\n\x11\x46\x65tchSourceBranch\x12 .gitaly.FetchSourceBranchRequest\x1a!.gitaly.FetchSourceBranchResponse\"\x06\xfa\x97(\x02\x08\x01\x12\x39\n\x04\x46sck\x12\x13.gitaly.FsckRequest\x1a\x14.gitaly.FsckResponse\"\x06\xfa\x97(\x02\x08\x02\x12\x45\n\x08WriteRef\x12\x17.gitaly.WriteRefRequest\x1a\x18.gitaly.WriteRefResponse\"\x06\xfa\x97(\x02\x08\x01\x12T\n\rFindMergeBase\x12\x1c.gitaly.FindMergeBaseRequest\x1a\x1d.gitaly.FindMergeBaseResponse\"\x06\xfa\x97(\x02\x08\x02\x12K\n\nCreateFork\x12\x19.gitaly.CreateForkRequest\x1a\x1a.gitaly.CreateForkResponse\"\x06\xfa\x97(\x02\x08\x01\x12r\n\x17\x43reateRepositoryFromURL\x12&.gitaly.CreateRepositoryFromURLRequest\x1a\'.gitaly.CreateRepositoryFromURLResponse\"\x06\xfa\x97(\x02\x08\x01\x12S\n\x0c\x43reateBundle\x12\x1b.gitaly.CreateBundleRequest\x1a\x1c.gitaly.CreateBundleResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12v\n\x17\x43reateBundleFromRefList\x12&.gitaly.CreateBundleFromRefListRequest\x1a\'.gitaly.CreateBundleFromRefListResponse\"\x06\xfa\x97(\x02\x08\x02(\x01\x30\x01\x12`\n\x11GenerateBundleURI\x12 .gitaly.GenerateBundleURIRequest\x1a!.gitaly.GenerateBundleURIResponse\"\x06\xfa\x97(\x02\x08\x02\x12P\n\x0b\x46\x65tchBundle\x12\x1a.gitaly.FetchBundleRequest\x1a\x1b.gitaly.FetchBundleResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x12}\n\x1a\x43reateRepositoryFromBundle\x12).gitaly.CreateRepositoryFromBundleRequest\x1a*.gitaly.CreateRepositoryFromBundleResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x12J\n\tGetConfig\x12\x18.gitaly.GetConfigRequest\x1a\x19.gitaly.GetConfigResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12N\n\x0b\x46indLicense\x12\x1a.gitaly.FindLicenseRequest\x1a\x1b.gitaly.FindLicenseResponse\"\x06\xfa\x97(\x02\x08\x02\x12\x65\n\x11GetInfoAttributes\x12 .gitaly.GetInfoAttributesRequest\x1a!.gitaly.GetInfoAttributesResponse\"\t\x88\x02\x01\xfa\x97(\x02\x08\x02\x30\x01\x12`\n\x11\x43\x61lculateChecksum\x12 .gitaly.CalculateChecksumRequest\x1a!.gitaly.CalculateChecksumResponse\"\x06\xfa\x97(\x02\x08\x02\x12P\n\x0bGetSnapshot\x12\x1a.gitaly.GetSnapshotRequest\x1a\x1b.gitaly.GetSnapshotResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\x81\x01\n\x1c\x43reateRepositoryFromSnapshot\x12+.gitaly.CreateRepositoryFromSnapshotRequest\x1a,.gitaly.CreateRepositoryFromSnapshotResponse\"\x06\xfa\x97(\x02\x08\x01\x12V\n\rGetRawChanges\x12\x1c.gitaly.GetRawChangesRequest\x1a\x1d.gitaly.GetRawChangesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12k\n\x14SearchFilesByContent\x12#.gitaly.SearchFilesByContentRequest\x1a$.gitaly.SearchFilesByContentResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\x62\n\x11SearchFilesByName\x12 .gitaly.SearchFilesByNameRequest\x1a!.gitaly.SearchFilesByNameResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12h\n\x12RestoreCustomHooks\x12!.gitaly.RestoreCustomHooksRequest\x1a\".gitaly.RestoreCustomHooksResponse\"\t\x88\x02\x01\xfa\x97(\x02\x08\x01(\x01\x12Y\n\x0eSetCustomHooks\x12\x1d.gitaly.SetCustomHooksRequest\x1a\x1e.gitaly.SetCustomHooksResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x12\x65\n\x11\x42\x61\x63kupCustomHooks\x12 .gitaly.BackupCustomHooksRequest\x1a!.gitaly.BackupCustomHooksResponse\"\t\x88\x02\x01\xfa\x97(\x02\x08\x02\x30\x01\x12Y\n\x0eGetCustomHooks\x12\x1d.gitaly.GetCustomHooksRequest\x1a\x1e.gitaly.GetCustomHooksResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12o\n\x16GetObjectDirectorySize\x12%.gitaly.GetObjectDirectorySizeRequest\x1a&.gitaly.GetObjectDirectorySizeResponse\"\x06\xfa\x97(\x02\x08\x02\x12]\n\x10RemoveRepository\x12\x1f.gitaly.RemoveRepositoryRequest\x1a .gitaly.RemoveRepositoryResponse\"\x06\xfa\x97(\x02\x08\x01\x12\x66\n\x13ReplicateRepository\x12\".gitaly.ReplicateRepositoryRequest\x1a#.gitaly.ReplicateRepositoryResponse\"\x06\xfa\x97(\x02\x08\x01\x12\x63\n\x12OptimizeRepository\x12!.gitaly.OptimizeRepositoryRequest\x1a\".gitaly.OptimizeRepositoryResponse\"\x06\xfa\x97(\x02\x08\x03\x12r\n\x17PruneUnreachableObjects\x12&.gitaly.PruneUnreachableObjectsRequest\x1a\'.gitaly.PruneUnreachableObjectsResponse\"\x06\xfa\x97(\x02\x08\x03\x12]\n\x10\x42\x61\x63kupRepository\x12\x1f.gitaly.BackupRepositoryRequest\x1a .gitaly.BackupRepositoryResponse\"\x06\xfa\x97(\x02\x08\x02\x12`\n\x11RestoreRepository\x12 .gitaly.RestoreRepositoryRequest\x1a!.gitaly.RestoreRepositoryResponse\"\x06\xfa\x97(\x02\x08\x01\x12`\n\x11GetFileAttributes\x12 .gitaly.GetFileAttributesRequest\x1a!.gitaly.GetFileAttributesResponse\"\x06\xfa\x97(\x02\x08\x02\x12M\n\nFastExport\x12\x19.gitaly.FastExportRequest\x1a\x1a.gitaly.FastExportResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'repository_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _REPOSITORYEXISTSREQUEST.fields_by_name['repository']._options = None
  _REPOSITORYEXISTSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REPOSITORYSIZEREQUEST.fields_by_name['repository']._options = None
  _REPOSITORYSIZEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REPOSITORYINFOREQUEST.fields_by_name['repository']._options = None
  _REPOSITORYINFOREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _OBJECTSSIZEREQUEST.fields_by_name['repository']._options = None
  _OBJECTSSIZEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _OBJECTFORMATREQUEST.fields_by_name['repository']._options = None
  _OBJECTFORMATREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _APPLYGITATTRIBUTESREQUEST.fields_by_name['repository']._options = None
  _APPLYGITATTRIBUTESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FETCHBUNDLEREQUEST.fields_by_name['repository']._options = None
  _FETCHBUNDLEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FETCHREMOTEREQUEST.fields_by_name['repository']._options = None
  _FETCHREMOTEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEREPOSITORYREQUEST.fields_by_name['repository']._options = None
  _CREATEREPOSITORYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETARCHIVEREQUEST.fields_by_name['repository']._options = None
  _GETARCHIVEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _HASLOCALBRANCHESREQUEST.fields_by_name['repository']._options = None
  _HASLOCALBRANCHESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FETCHSOURCEBRANCHREQUEST.fields_by_name['repository']._options = None
  _FETCHSOURCEBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FSCKREQUEST.fields_by_name['repository']._options = None
  _FSCKREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _WRITEREFREQUEST.fields_by_name['repository']._options = None
  _WRITEREFREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDMERGEBASEREQUEST.fields_by_name['repository']._options = None
  _FINDMERGEBASEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEFORKREQUEST.fields_by_name['repository']._options = None
  _CREATEFORKREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEREPOSITORYFROMURLREQUEST.fields_by_name['repository']._options = None
  _CREATEREPOSITORYFROMURLREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEBUNDLEREQUEST.fields_by_name['repository']._options = None
  _CREATEBUNDLEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEBUNDLEFROMREFLISTREQUEST.fields_by_name['repository']._options = None
  _CREATEBUNDLEFROMREFLISTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GENERATEBUNDLEURIREQUEST.fields_by_name['repository']._options = None
  _GENERATEBUNDLEURIREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETCONFIGREQUEST.fields_by_name['repository']._options = None
  _GETCONFIGREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _RESTORECUSTOMHOOKSREQUEST.fields_by_name['repository']._options = None
  _RESTORECUSTOMHOOKSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _SETCUSTOMHOOKSREQUEST.fields_by_name['repository']._options = None
  _SETCUSTOMHOOKSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _BACKUPCUSTOMHOOKSREQUEST.fields_by_name['repository']._options = None
  _BACKUPCUSTOMHOOKSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETCUSTOMHOOKSREQUEST.fields_by_name['repository']._options = None
  _GETCUSTOMHOOKSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEREPOSITORYFROMBUNDLEREQUEST.fields_by_name['repository']._options = None
  _CREATEREPOSITORYFROMBUNDLEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDLICENSEREQUEST.fields_by_name['repository']._options = None
  _FINDLICENSEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETINFOATTRIBUTESREQUEST.fields_by_name['repository']._options = None
  _GETINFOATTRIBUTESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CALCULATECHECKSUMREQUEST.fields_by_name['repository']._options = None
  _CALCULATECHECKSUMREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETSNAPSHOTREQUEST.fields_by_name['repository']._options = None
  _GETSNAPSHOTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEREPOSITORYFROMSNAPSHOTREQUEST.fields_by_name['repository']._options = None
  _CREATEREPOSITORYFROMSNAPSHOTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETRAWCHANGESREQUEST.fields_by_name['repository']._options = None
  _GETRAWCHANGESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _SEARCHFILESBYNAMEREQUEST.fields_by_name['repository']._options = None
  _SEARCHFILESBYNAMEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _SEARCHFILESBYCONTENTREQUEST.fields_by_name['repository']._options = None
  _SEARCHFILESBYCONTENTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETOBJECTDIRECTORYSIZEREQUEST.fields_by_name['repository']._options = None
  _GETOBJECTDIRECTORYSIZEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REMOVEREPOSITORYREQUEST.fields_by_name['repository']._options = None
  _REMOVEREPOSITORYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REPLICATEREPOSITORYREQUEST.fields_by_name['repository']._options = None
  _REPLICATEREPOSITORYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _OPTIMIZEREPOSITORYREQUEST.fields_by_name['repository']._options = None
  _OPTIMIZEREPOSITORYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _PRUNEUNREACHABLEOBJECTSREQUEST.fields_by_name['repository']._options = None
  _PRUNEUNREACHABLEOBJECTSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _BACKUPREPOSITORYREQUEST.fields_by_name['repository']._options = None
  _BACKUPREPOSITORYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _RESTOREREPOSITORYREQUEST.fields_by_name['repository']._options = None
  _RESTOREREPOSITORYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETFILEATTRIBUTESREQUEST.fields_by_name['repository']._options = None
  _GETFILEATTRIBUTESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FASTEXPORTREQUEST.fields_by_name['repository']._options = None
  _FASTEXPORTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REPOSITORYSERVICE.methods_by_name['RepositoryExists']._options = None
  _REPOSITORYSERVICE.methods_by_name['RepositoryExists']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['RepositorySize']._options = None
  _REPOSITORYSERVICE.methods_by_name['RepositorySize']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['RepositoryInfo']._options = None
  _REPOSITORYSERVICE.methods_by_name['RepositoryInfo']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['ObjectsSize']._options = None
  _REPOSITORYSERVICE.methods_by_name['ObjectsSize']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['ObjectFormat']._options = None
  _REPOSITORYSERVICE.methods_by_name['ObjectFormat']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['ApplyGitattributes']._options = None
  _REPOSITORYSERVICE.methods_by_name['ApplyGitattributes']._serialized_options = b'\210\002\001\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['FetchRemote']._options = None
  _REPOSITORYSERVICE.methods_by_name['FetchRemote']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['CreateRepository']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateRepository']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['GetArchive']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetArchive']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['HasLocalBranches']._options = None
  _REPOSITORYSERVICE.methods_by_name['HasLocalBranches']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['FetchSourceBranch']._options = None
  _REPOSITORYSERVICE.methods_by_name['FetchSourceBranch']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['Fsck']._options = None
  _REPOSITORYSERVICE.methods_by_name['Fsck']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['WriteRef']._options = None
  _REPOSITORYSERVICE.methods_by_name['WriteRef']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['FindMergeBase']._options = None
  _REPOSITORYSERVICE.methods_by_name['FindMergeBase']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['CreateFork']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateFork']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['CreateRepositoryFromURL']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateRepositoryFromURL']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['CreateBundle']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateBundle']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['CreateBundleFromRefList']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateBundleFromRefList']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['GenerateBundleURI']._options = None
  _REPOSITORYSERVICE.methods_by_name['GenerateBundleURI']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['FetchBundle']._options = None
  _REPOSITORYSERVICE.methods_by_name['FetchBundle']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['CreateRepositoryFromBundle']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateRepositoryFromBundle']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['GetConfig']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetConfig']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['FindLicense']._options = None
  _REPOSITORYSERVICE.methods_by_name['FindLicense']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['GetInfoAttributes']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetInfoAttributes']._serialized_options = b'\210\002\001\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['CalculateChecksum']._options = None
  _REPOSITORYSERVICE.methods_by_name['CalculateChecksum']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['GetSnapshot']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetSnapshot']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['CreateRepositoryFromSnapshot']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateRepositoryFromSnapshot']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['GetRawChanges']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetRawChanges']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['SearchFilesByContent']._options = None
  _REPOSITORYSERVICE.methods_by_name['SearchFilesByContent']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['SearchFilesByName']._options = None
  _REPOSITORYSERVICE.methods_by_name['SearchFilesByName']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['RestoreCustomHooks']._options = None
  _REPOSITORYSERVICE.methods_by_name['RestoreCustomHooks']._serialized_options = b'\210\002\001\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['SetCustomHooks']._options = None
  _REPOSITORYSERVICE.methods_by_name['SetCustomHooks']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['BackupCustomHooks']._options = None
  _REPOSITORYSERVICE.methods_by_name['BackupCustomHooks']._serialized_options = b'\210\002\001\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['GetCustomHooks']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetCustomHooks']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['GetObjectDirectorySize']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetObjectDirectorySize']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['RemoveRepository']._options = None
  _REPOSITORYSERVICE.methods_by_name['RemoveRepository']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['ReplicateRepository']._options = None
  _REPOSITORYSERVICE.methods_by_name['ReplicateRepository']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['OptimizeRepository']._options = None
  _REPOSITORYSERVICE.methods_by_name['OptimizeRepository']._serialized_options = b'\372\227(\002\010\003'
  _REPOSITORYSERVICE.methods_by_name['PruneUnreachableObjects']._options = None
  _REPOSITORYSERVICE.methods_by_name['PruneUnreachableObjects']._serialized_options = b'\372\227(\002\010\003'
  _REPOSITORYSERVICE.methods_by_name['BackupRepository']._options = None
  _REPOSITORYSERVICE.methods_by_name['BackupRepository']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['RestoreRepository']._options = None
  _REPOSITORYSERVICE.methods_by_name['RestoreRepository']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['GetFileAttributes']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetFileAttributes']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['FastExport']._options = None
  _REPOSITORYSERVICE.methods_by_name['FastExport']._serialized_options = b'\372\227(\002\010\002'
  _globals['_REPOSITORYEXISTSREQUEST']._serialized_start=54
  _globals['_REPOSITORYEXISTSREQUEST']._serialized_end=125
  _globals['_REPOSITORYEXISTSRESPONSE']._serialized_start=127
  _globals['_REPOSITORYEXISTSRESPONSE']._serialized_end=169
  _globals['_REPOSITORYSIZEREQUEST']._serialized_start=171
  _globals['_REPOSITORYSIZEREQUEST']._serialized_end=240
  _globals['_REPOSITORYSIZERESPONSE']._serialized_start=242
  _globals['_REPOSITORYSIZERESPONSE']._serialized_end=280
  _globals['_REPOSITORYINFOREQUEST']._serialized_start=282
  _globals['_REPOSITORYINFOREQUEST']._serialized_end=351
  _globals['_REPOSITORYINFORESPONSE']._serialized_start=354
  _globals['_REPOSITORYINFORESPONSE']._serialized_end=669
  _globals['_REPOSITORYINFORESPONSE_REFERENCESINFO']._serialized_start=522
  _globals['_REPOSITORYINFORESPONSE_REFERENCESINFO']._serialized_end=580
  _globals['_REPOSITORYINFORESPONSE_OBJECTSINFO']._serialized_start=582
  _globals['_REPOSITORYINFORESPONSE_OBJECTSINFO']._serialized_end=669
  _globals['_OBJECTSSIZEREQUEST']._serialized_start=671
  _globals['_OBJECTSSIZEREQUEST']._serialized_end=756
  _globals['_OBJECTSSIZERESPONSE']._serialized_start=758
  _globals['_OBJECTSSIZERESPONSE']._serialized_end=793
  _globals['_OBJECTFORMATREQUEST']._serialized_start=795
  _globals['_OBJECTFORMATREQUEST']._serialized_end=862
  _globals['_OBJECTFORMATRESPONSE']._serialized_start=864
  _globals['_OBJECTFORMATRESPONSE']._serialized_end=924
  _globals['_APPLYGITATTRIBUTESREQUEST']._serialized_start=926
  _globals['_APPLYGITATTRIBUTESREQUEST']._serialized_end=1017
  _globals['_APPLYGITATTRIBUTESRESPONSE']._serialized_start=1019
  _globals['_APPLYGITATTRIBUTESRESPONSE']._serialized_end=1047
  _globals['_FETCHBUNDLEREQUEST']._serialized_start=1049
  _globals['_FETCHBUNDLEREQUEST']._serialized_end=1150
  _globals['_FETCHBUNDLERESPONSE']._serialized_start=1152
  _globals['_FETCHBUNDLERESPONSE']._serialized_end=1173
  _globals['_FETCHREMOTEREQUEST']._serialized_start=1176
  _globals['_FETCHREMOTEREQUEST']._serialized_end=1434
  _globals['_FETCHREMOTERESPONSE']._serialized_start=1436
  _globals['_FETCHREMOTERESPONSE']._serialized_end=1479
  _globals['_CREATEREPOSITORYREQUEST']._serialized_start=1482
  _globals['_CREATEREPOSITORYREQUEST']._serialized_end=1622
  _globals['_CREATEREPOSITORYRESPONSE']._serialized_start=1624
  _globals['_CREATEREPOSITORYRESPONSE']._serialized_end=1650
  _globals['_GETARCHIVEREQUEST']._serialized_start=1653
  _globals['_GETARCHIVEREQUEST']._serialized_end=1934
  _globals['_GETARCHIVEREQUEST_FORMAT']._serialized_start=1883
  _globals['_GETARCHIVEREQUEST_FORMAT']._serialized_end=1934
  _globals['_GETARCHIVERESPONSE']._serialized_start=1936
  _globals['_GETARCHIVERESPONSE']._serialized_end=1970
  _globals['_HASLOCALBRANCHESREQUEST']._serialized_start=1972
  _globals['_HASLOCALBRANCHESREQUEST']._serialized_end=2043
  _globals['_HASLOCALBRANCHESRESPONSE']._serialized_start=2045
  _globals['_HASLOCALBRANCHESRESPONSE']._serialized_end=2086
  _globals['_FETCHSOURCEBRANCHREQUEST']._serialized_start=2089
  _globals['_FETCHSOURCEBRANCHREQUEST']._serialized_end=2284
  _globals['_FETCHSOURCEBRANCHRESPONSE']._serialized_start=2286
  _globals['_FETCHSOURCEBRANCHRESPONSE']._serialized_end=2329
  _globals['_FSCKREQUEST']._serialized_start=2331
  _globals['_FSCKREQUEST']._serialized_end=2390
  _globals['_FSCKRESPONSE']._serialized_start=2392
  _globals['_FSCKRESPONSE']._serialized_end=2421
  _globals['_WRITEREFREQUEST']._serialized_start=2424
  _globals['_WRITEREFREQUEST']._serialized_end=2559
  _globals['_WRITEREFRESPONSE']._serialized_start=2561
  _globals['_WRITEREFRESPONSE']._serialized_end=2585
  _globals['_FINDMERGEBASEREQUEST']._serialized_start=2587
  _globals['_FINDMERGEBASEREQUEST']._serialized_end=2674
  _globals['_FINDMERGEBASERESPONSE']._serialized_start=2676
  _globals['_FINDMERGEBASERESPONSE']._serialized_end=2713
  _globals['_CREATEFORKREQUEST']._serialized_start=2716
  _globals['_CREATEFORKREQUEST']._serialized_end=2846
  _globals['_CREATEFORKRESPONSE']._serialized_start=2848
  _globals['_CREATEFORKRESPONSE']._serialized_end=2868
  _globals['_CREATEREPOSITORYFROMURLREQUEST']._serialized_start=2871
  _globals['_CREATEREPOSITORYFROMURLREQUEST']._serialized_end=3056
  _globals['_CREATEREPOSITORYFROMURLRESPONSE']._serialized_start=3058
  _globals['_CREATEREPOSITORYFROMURLRESPONSE']._serialized_end=3091
  _globals['_CREATEBUNDLEREQUEST']._serialized_start=3093
  _globals['_CREATEBUNDLEREQUEST']._serialized_end=3160
  _globals['_CREATEBUNDLERESPONSE']._serialized_start=3162
  _globals['_CREATEBUNDLERESPONSE']._serialized_end=3198
  _globals['_CREATEBUNDLEFROMREFLISTREQUEST']._serialized_start=3200
  _globals['_CREATEBUNDLEFROMREFLISTREQUEST']._serialized_end=3296
  _globals['_CREATEBUNDLEFROMREFLISTRESPONSE']._serialized_start=3298
  _globals['_CREATEBUNDLEFROMREFLISTRESPONSE']._serialized_end=3345
  _globals['_GENERATEBUNDLEURIREQUEST']._serialized_start=3347
  _globals['_GENERATEBUNDLEURIREQUEST']._serialized_end=3419
  _globals['_GENERATEBUNDLEURIRESPONSE']._serialized_start=3421
  _globals['_GENERATEBUNDLEURIRESPONSE']._serialized_end=3448
  _globals['_GETCONFIGREQUEST']._serialized_start=3450
  _globals['_GETCONFIGREQUEST']._serialized_end=3514
  _globals['_GETCONFIGRESPONSE']._serialized_start=3516
  _globals['_GETCONFIGRESPONSE']._serialized_end=3549
  _globals['_RESTORECUSTOMHOOKSREQUEST']._serialized_start=3551
  _globals['_RESTORECUSTOMHOOKSREQUEST']._serialized_end=3638
  _globals['_SETCUSTOMHOOKSREQUEST']._serialized_start=3640
  _globals['_SETCUSTOMHOOKSREQUEST']._serialized_end=3723
  _globals['_RESTORECUSTOMHOOKSRESPONSE']._serialized_start=3725
  _globals['_RESTORECUSTOMHOOKSRESPONSE']._serialized_end=3753
  _globals['_SETCUSTOMHOOKSRESPONSE']._serialized_start=3755
  _globals['_SETCUSTOMHOOKSRESPONSE']._serialized_end=3779
  _globals['_BACKUPCUSTOMHOOKSREQUEST']._serialized_start=3781
  _globals['_BACKUPCUSTOMHOOKSREQUEST']._serialized_end=3853
  _globals['_GETCUSTOMHOOKSREQUEST']._serialized_start=3855
  _globals['_GETCUSTOMHOOKSREQUEST']._serialized_end=3924
  _globals['_BACKUPCUSTOMHOOKSRESPONSE']._serialized_start=3926
  _globals['_BACKUPCUSTOMHOOKSRESPONSE']._serialized_end=3967
  _globals['_GETCUSTOMHOOKSRESPONSE']._serialized_start=3969
  _globals['_GETCUSTOMHOOKSRESPONSE']._serialized_end=4007
  _globals['_CREATEREPOSITORYFROMBUNDLEREQUEST']._serialized_start=4009
  _globals['_CREATEREPOSITORYFROMBUNDLEREQUEST']._serialized_end=4104
  _globals['_CREATEREPOSITORYFROMBUNDLERESPONSE']._serialized_start=4106
  _globals['_CREATEREPOSITORYFROMBUNDLERESPONSE']._serialized_end=4142
  _globals['_FINDLICENSEREQUEST']._serialized_start=4144
  _globals['_FINDLICENSEREQUEST']._serialized_end=4210
  _globals['_FINDLICENSERESPONSE']._serialized_start=4213
  _globals['_FINDLICENSERESPONSE']._serialized_end=4353
  _globals['_GETINFOATTRIBUTESREQUEST']._serialized_start=4355
  _globals['_GETINFOATTRIBUTESREQUEST']._serialized_end=4427
  _globals['_GETINFOATTRIBUTESRESPONSE']._serialized_start=4429
  _globals['_GETINFOATTRIBUTESRESPONSE']._serialized_end=4476
  _globals['_CALCULATECHECKSUMREQUEST']._serialized_start=4478
  _globals['_CALCULATECHECKSUMREQUEST']._serialized_end=4550
  _globals['_CALCULATECHECKSUMRESPONSE']._serialized_start=4552
  _globals['_CALCULATECHECKSUMRESPONSE']._serialized_end=4597
  _globals['_GETSNAPSHOTREQUEST']._serialized_start=4599
  _globals['_GETSNAPSHOTREQUEST']._serialized_end=4665
  _globals['_GETSNAPSHOTRESPONSE']._serialized_start=4667
  _globals['_GETSNAPSHOTRESPONSE']._serialized_end=4702
  _globals['_CREATEREPOSITORYFROMSNAPSHOTREQUEST']._serialized_start=4705
  _globals['_CREATEREPOSITORYFROMSNAPSHOTREQUEST']._serialized_end=4868
  _globals['_CREATEREPOSITORYFROMSNAPSHOTRESPONSE']._serialized_start=4870
  _globals['_CREATEREPOSITORYFROMSNAPSHOTRESPONSE']._serialized_end=4908
  _globals['_GETRAWCHANGESREQUEST']._serialized_start=4910
  _globals['_GETRAWCHANGESREQUEST']._serialized_end=5022
  _globals['_GETRAWCHANGESRESPONSE']._serialized_start=5025
  _globals['_GETRAWCHANGESRESPONSE']._serialized_end=5469
  _globals['_GETRAWCHANGESRESPONSE_RAWCHANGE']._serialized_start=5113
  _globals['_GETRAWCHANGESRESPONSE_RAWCHANGE']._serialized_end=5469
  _globals['_GETRAWCHANGESRESPONSE_RAWCHANGE_OPERATION']._serialized_start=5311
  _globals['_GETRAWCHANGESRESPONSE_RAWCHANGE_OPERATION']._serialized_end=5416
  _globals['_SEARCHFILESBYNAMEREQUEST']._serialized_start=5472
  _globals['_SEARCHFILESBYNAMEREQUEST']._serialized_end=5619
  _globals['_SEARCHFILESBYNAMERESPONSE']._serialized_start=5621
  _globals['_SEARCHFILESBYNAMERESPONSE']._serialized_end=5663
  _globals['_SEARCHFILESBYCONTENTREQUEST']._serialized_start=5665
  _globals['_SEARCHFILESBYCONTENTREQUEST']._serialized_end=5792
  _globals['_SEARCHFILESBYCONTENTRESPONSE']._serialized_start=5794
  _globals['_SEARCHFILESBYCONTENTRESPONSE']._serialized_end=5881
  _globals['_REMOTE']._serialized_start=5884
  _globals['_REMOTE']._serialized_end=6019
  _globals['_GETOBJECTDIRECTORYSIZEREQUEST']._serialized_start=6021
  _globals['_GETOBJECTDIRECTORYSIZEREQUEST']._serialized_end=6098
  _globals['_GETOBJECTDIRECTORYSIZERESPONSE']._serialized_start=6100
  _globals['_GETOBJECTDIRECTORYSIZERESPONSE']._serialized_end=6146
  _globals['_REMOVEREPOSITORYREQUEST']._serialized_start=6148
  _globals['_REMOVEREPOSITORYREQUEST']._serialized_end=6219
  _globals['_REMOVEREPOSITORYRESPONSE']._serialized_start=6221
  _globals['_REMOVEREPOSITORYRESPONSE']._serialized_end=6247
  _globals['_REPLICATEREPOSITORYREQUEST']._serialized_start=6249
  _globals['_REPLICATEREPOSITORYREQUEST']._serialized_end=6359
  _globals['_REPLICATEREPOSITORYRESPONSE']._serialized_start=6361
  _globals['_REPLICATEREPOSITORYRESPONSE']._serialized_end=6390
  _globals['_OPTIMIZEREPOSITORYREQUEST']._serialized_start=6393
  _globals['_OPTIMIZEREPOSITORYREQUEST']._serialized_end=6612
  _globals['_OPTIMIZEREPOSITORYREQUEST_STRATEGY']._serialized_start=6530
  _globals['_OPTIMIZEREPOSITORYREQUEST_STRATEGY']._serialized_end=6612
  _globals['_OPTIMIZEREPOSITORYRESPONSE']._serialized_start=6614
  _globals['_OPTIMIZEREPOSITORYRESPONSE']._serialized_end=6642
  _globals['_PRUNEUNREACHABLEOBJECTSREQUEST']._serialized_start=6644
  _globals['_PRUNEUNREACHABLEOBJECTSREQUEST']._serialized_end=6722
  _globals['_PRUNEUNREACHABLEOBJECTSRESPONSE']._serialized_start=6724
  _globals['_PRUNEUNREACHABLEOBJECTSRESPONSE']._serialized_end=6757
  _globals['_BACKUPREPOSITORYREQUEST']._serialized_start=6760
  _globals['_BACKUPREPOSITORYREQUEST']._serialized_end=6918
  _globals['_BACKUPREPOSITORYRESPONSE']._serialized_start=6920
  _globals['_BACKUPREPOSITORYRESPONSE']._serialized_end=6962
  _globals['_BACKUPREPOSITORYRESPONSE_SKIPPEDERROR']._serialized_start=6948
  _globals['_BACKUPREPOSITORYRESPONSE_SKIPPEDERROR']._serialized_end=6962
  _globals['_RESTOREREPOSITORYREQUEST']._serialized_start=6965
  _globals['_RESTOREREPOSITORYREQUEST']._serialized_end=7126
  _globals['_RESTOREREPOSITORYRESPONSE']._serialized_start=7128
  _globals['_RESTOREREPOSITORYRESPONSE']._serialized_end=7171
  _globals['_RESTOREREPOSITORYRESPONSE_SKIPPEDERROR']._serialized_start=6948
  _globals['_RESTOREREPOSITORYRESPONSE_SKIPPEDERROR']._serialized_end=6962
  _globals['_GETFILEATTRIBUTESREQUEST']._serialized_start=7173
  _globals['_GETFILEATTRIBUTESREQUEST']._serialized_end=7298
  _globals['_GETFILEATTRIBUTESRESPONSE']._serialized_start=7301
  _globals['_GETFILEATTRIBUTESRESPONSE']._serialized_end=7467
  _globals['_GETFILEATTRIBUTESRESPONSE_ATTRIBUTEINFO']._serialized_start=7404
  _globals['_GETFILEATTRIBUTESRESPONSE_ATTRIBUTEINFO']._serialized_end=7467
  _globals['_FASTEXPORTREQUEST']._serialized_start=7469
  _globals['_FASTEXPORTREQUEST']._serialized_end=7534
  _globals['_FASTEXPORTRESPONSE']._serialized_start=7536
  _globals['_FASTEXPORTRESPONSE']._serialized_end=7570
  _globals['_REPOSITORYSERVICE']._serialized_start=7573
  _globals['_REPOSITORYSERVICE']._serialized_end=11660
# @@protoc_insertion_point(module_scope)
