# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: errors.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x65rrors.proto\x12\x06gitaly\x1a\x1egoogle/protobuf/duration.proto\"]\n\x10\x41\x63\x63\x65ssCheckError\x12\x15\n\rerror_message\x18\x01 \x01(\t\x12\x10\n\x08protocol\x18\x02 \x01(\t\x12\x0f\n\x07user_id\x18\x03 \x01(\t\x12\x0f\n\x07\x63hanges\x18\x04 \x01(\x0c\"\xae\x02\n\nIndexError\x12\x0c\n\x04path\x18\x01 \x01(\x0c\x12\x30\n\nerror_type\x18\x02 \x01(\x0e\x32\x1c.gitaly.IndexError.ErrorType\"\xdf\x01\n\tErrorType\x12\x1a\n\x16\x45RROR_TYPE_UNSPECIFIED\x10\x00\x12\x19\n\x15\x45RROR_TYPE_EMPTY_PATH\x10\x01\x12\x1b\n\x17\x45RROR_TYPE_INVALID_PATH\x10\x02\x12\x1f\n\x1b\x45RROR_TYPE_DIRECTORY_EXISTS\x10\x03\x12\"\n\x1e\x45RROR_TYPE_DIRECTORY_TRAVERSAL\x10\x04\x12\x1a\n\x16\x45RROR_TYPE_FILE_EXISTS\x10\x05\x12\x1d\n\x19\x45RROR_TYPE_FILE_NOT_FOUND\x10\x06\"%\n\x15InvalidRefFormatError\x12\x0c\n\x04refs\x18\x02 \x03(\x0c\"C\n\x10NotAncestorError\x12\x17\n\x0fparent_revision\x18\x01 \x01(\x0c\x12\x16\n\x0e\x63hild_revision\x18\x02 \x01(\x0c\"\x1c\n\x1a\x43hangesAlreadyAppliedError\"O\n\x12MergeConflictError\x12\x19\n\x11\x63onflicting_files\x18\x01 \x03(\x0c\x12\x1e\n\x16\x63onflicting_commit_ids\x18\x02 \x03(\t\"%\n\x15ReferencesLockedError\x12\x0c\n\x04refs\x18\x01 \x03(\x0c\";\n\x14ReferenceExistsError\x12\x16\n\x0ereference_name\x18\x01 \x01(\x0c\x12\x0b\n\x03oid\x18\x02 \x01(\t\"0\n\x16ReferenceNotFoundError\x12\x16\n\x0ereference_name\x18\x01 \x01(\x0c\"k\n\x1bReferenceStateMismatchError\x12\x16\n\x0ereference_name\x18\x01 \x01(\x0c\x12\x1a\n\x12\x65xpected_object_id\x18\x02 \x01(\x0c\x12\x18\n\x10\x61\x63tual_object_id\x18\x03 \x01(\x0c\"P\n\x14ReferenceUpdateError\x12\x16\n\x0ereference_name\x18\x01 \x01(\x0c\x12\x0f\n\x07old_oid\x18\x02 \x01(\t\x12\x0f\n\x07new_oid\x18\x03 \x01(\t\"(\n\x14ResolveRevisionError\x12\x10\n\x08revision\x18\x01 \x01(\x0c\"S\n\nLimitError\x12\x15\n\rerror_message\x18\x01 \x01(\t\x12.\n\x0bretry_after\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\"\xd8\x01\n\x0f\x43ustomHookError\x12\x0e\n\x06stdout\x18\x01 \x01(\x0c\x12\x0e\n\x06stderr\x18\x02 \x01(\x0c\x12\x33\n\thook_type\x18\x03 \x01(\x0e\x32 .gitaly.CustomHookError.HookType\"p\n\x08HookType\x12\x19\n\x15HOOK_TYPE_UNSPECIFIED\x10\x00\x12\x18\n\x14HOOK_TYPE_PRERECEIVE\x10\x01\x12\x14\n\x10HOOK_TYPE_UPDATE\x10\x02\x12\x19\n\x15HOOK_TYPE_POSTRECEIVE\x10\x03\"\xf8\x01\n\tPathError\x12\x0c\n\x04path\x18\x01 \x01(\x0c\x12/\n\nerror_type\x18\x02 \x01(\x0e\x32\x1b.gitaly.PathError.ErrorType\"\xab\x01\n\tErrorType\x12\x1a\n\x16\x45RROR_TYPE_UNSPECIFIED\x10\x00\x12\x19\n\x15\x45RROR_TYPE_EMPTY_PATH\x10\x01\x12/\n+ERROR_TYPE_RELATIVE_PATH_ESCAPES_REPOSITORY\x10\x02\x12\x1c\n\x18\x45RROR_TYPE_ABSOLUTE_PATH\x10\x03\x12\x18\n\x14\x45RROR_TYPE_LONG_PATH\x10\x04\"!\n\x11PathNotFoundError\x12\x0c\n\x04path\x18\x01 \x01(\x0c\",\n\x17\x41mbiguousReferenceError\x12\x11\n\treference\x18\x01 \x01(\x0c\"!\n\x0e\x42\x61\x64ObjectError\x12\x0f\n\x07\x62\x61\x64_oid\x18\x01 \x01(\x0c\"%\n\x14InvalidRevisionRange\x12\r\n\x05range\x18\x01 \x01(\x0c\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'errors_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _globals['_ACCESSCHECKERROR']._serialized_start=56
  _globals['_ACCESSCHECKERROR']._serialized_end=149
  _globals['_INDEXERROR']._serialized_start=152
  _globals['_INDEXERROR']._serialized_end=454
  _globals['_INDEXERROR_ERRORTYPE']._serialized_start=231
  _globals['_INDEXERROR_ERRORTYPE']._serialized_end=454
  _globals['_INVALIDREFFORMATERROR']._serialized_start=456
  _globals['_INVALIDREFFORMATERROR']._serialized_end=493
  _globals['_NOTANCESTORERROR']._serialized_start=495
  _globals['_NOTANCESTORERROR']._serialized_end=562
  _globals['_CHANGESALREADYAPPLIEDERROR']._serialized_start=564
  _globals['_CHANGESALREADYAPPLIEDERROR']._serialized_end=592
  _globals['_MERGECONFLICTERROR']._serialized_start=594
  _globals['_MERGECONFLICTERROR']._serialized_end=673
  _globals['_REFERENCESLOCKEDERROR']._serialized_start=675
  _globals['_REFERENCESLOCKEDERROR']._serialized_end=712
  _globals['_REFERENCEEXISTSERROR']._serialized_start=714
  _globals['_REFERENCEEXISTSERROR']._serialized_end=773
  _globals['_REFERENCENOTFOUNDERROR']._serialized_start=775
  _globals['_REFERENCENOTFOUNDERROR']._serialized_end=823
  _globals['_REFERENCESTATEMISMATCHERROR']._serialized_start=825
  _globals['_REFERENCESTATEMISMATCHERROR']._serialized_end=932
  _globals['_REFERENCEUPDATEERROR']._serialized_start=934
  _globals['_REFERENCEUPDATEERROR']._serialized_end=1014
  _globals['_RESOLVEREVISIONERROR']._serialized_start=1016
  _globals['_RESOLVEREVISIONERROR']._serialized_end=1056
  _globals['_LIMITERROR']._serialized_start=1058
  _globals['_LIMITERROR']._serialized_end=1141
  _globals['_CUSTOMHOOKERROR']._serialized_start=1144
  _globals['_CUSTOMHOOKERROR']._serialized_end=1360
  _globals['_CUSTOMHOOKERROR_HOOKTYPE']._serialized_start=1248
  _globals['_CUSTOMHOOKERROR_HOOKTYPE']._serialized_end=1360
  _globals['_PATHERROR']._serialized_start=1363
  _globals['_PATHERROR']._serialized_end=1611
  _globals['_PATHERROR_ERRORTYPE']._serialized_start=1440
  _globals['_PATHERROR_ERRORTYPE']._serialized_end=1611
  _globals['_PATHNOTFOUNDERROR']._serialized_start=1613
  _globals['_PATHNOTFOUNDERROR']._serialized_end=1646
  _globals['_AMBIGUOUSREFERENCEERROR']._serialized_start=1648
  _globals['_AMBIGUOUSREFERENCEERROR']._serialized_end=1692
  _globals['_BADOBJECTERROR']._serialized_start=1694
  _globals['_BADOBJECTERROR']._serialized_end=1727
  _globals['_INVALIDREVISIONRANGE']._serialized_start=1729
  _globals['_INVALIDREVISIONRANGE']._serialized_end=1766
# @@protoc_insertion_point(module_scope)
