# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mercurial-operations.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import errors_pb2 as errors__pb2
from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import operations_pb2 as operations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1amercurial-operations.proto\x12\x07hgitaly\x1a\x0c\x65rrors.proto\x1a\nlint.proto\x1a\x0cshared.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x10operations.proto\"\x94\x01\n\x14MergeAnalysisRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x17\n\x0fsource_revision\x18\x02 \x01(\x0c\x12\x17\n\x0ftarget_revision\x18\x03 \x01(\x0c\x12\x1c\n\x14skip_conflicts_check\x18\x04 \x01(\x08\"\xad\x02\n\x15MergeAnalysisResponse\x12\x17\n\x0fis_fast_forward\x18\x01 \x01(\x08\x12\x1f\n\x17has_obsolete_changesets\x18\x02 \x01(\x08\x12\x1f\n\x17has_unstable_changesets\x18\x03 \x01(\x08\x12\x15\n\rhas_conflicts\x18\x04 \x01(\x08\x12\x18\n\x10target_is_public\x18\x05 \x01(\x08\x12\x16\n\x0etarget_node_id\x18\x06 \x01(\t\x12\x15\n\rtarget_branch\x18\x07 \x01(\x0c\x12\x14\n\x0ctarget_topic\x18\x08 \x01(\x0c\x12\x16\n\x0esource_node_id\x18\t \x01(\t\x12\x15\n\rsource_branch\x18\n \x01(\x0c\x12\x14\n\x0csource_topic\x18\x0b \x01(\x0c\"\xad\x01\n\x17PublishChangesetRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12/\n\x08hg_perms\x18\x03 \x01(\x0e\x32\x1d.hgitaly.MercurialPermissions\x12\x17\n\x0fgitlab_revision\x18\x04 \x01(\x0c\"\x1a\n\x18PublishChangesetResponse\"P\n\x15PublishChangesetError\x12.\n\x0bgitlab_hook\x18\x01 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x42\x07\n\x05\x65rror\"\xa1\x02\n\x12MergeBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12/\n\x08hg_perms\x18\x03 \x01(\x0e\x32\x1d.hgitaly.MercurialPermissions\x12\x11\n\tcommit_id\x18\x04 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x05 \x01(\x0c\x12\x0f\n\x07message\x18\x06 \x01(\x0c\x12-\n\ttimestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\x08 \x01(\t\x12\x13\n\x0bsemi_linear\x18\t \x01(\x08\"K\n\x13MergeBranchResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdate\"\xe7\x01\n\x10MergeBranchError\x12.\n\x0bgitlab_hook\x18\x01 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x12.\n\x08\x63onflict\x18\x02 \x01(\x0b\x32\x1a.gitaly.MergeConflictErrorH\x00\x12\x37\n\x0freference_check\x18\x03 \x01(\x0b\x32\x1c.gitaly.ReferenceUpdateErrorH\x00\x12\x31\n\tpre_check\x18\x04 \x01(\x0e\x32\x1c.hgitaly.PreCheckUpdateErrorH\x00\x42\x07\n\x05\x65rror\"\x9a\x01\n\rCensorRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x19\n\x11\x63hangeset_node_id\x18\x03 \x01(\t\x12\x11\n\tfile_path\x18\x04 \x01(\x0c\x12\x11\n\ttombstone\x18\x05 \x01(\x0c\"\x10\n\x0e\x43\x65nsorResponse*8\n\x14MercurialPermissions\x12\x08\n\x04READ\x10\x00\x12\t\n\x05WRITE\x10\x01\x12\x0b\n\x07PUBLISH\x10\x02*k\n\x13PreCheckUpdateError\x12\x0e\n\nNO_PROBLEM\x10\x00\x12\x14\n\x10NOT_FAST_FORWARD\x10\x01\x12\x16\n\x12OBSOLETE_CHANGESET\x10\x02\x12\x16\n\x12UNSTABLE_CHANGESET\x10\x03\x32\xea\x02\n\x1aMercurialOperationsService\x12V\n\rMergeAnalysis\x12\x1d.hgitaly.MergeAnalysisRequest\x1a\x1e.hgitaly.MergeAnalysisResponse\"\x06\xfa\x97(\x02\x08\x02\x12_\n\x10PublishChangeset\x12 .hgitaly.PublishChangesetRequest\x1a!.hgitaly.PublishChangesetResponse\"\x06\xfa\x97(\x02\x08\x01\x12\x41\n\x06\x43\x65nsor\x12\x16.hgitaly.CensorRequest\x1a\x17.hgitaly.CensorResponse\"\x06\xfa\x97(\x02\x08\x01\x12P\n\x0bMergeBranch\x12\x1b.hgitaly.MergeBranchRequest\x1a\x1c.hgitaly.MergeBranchResponse\"\x06\xfa\x97(\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mercurial_operations_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MERGEANALYSISREQUEST.fields_by_name['repository']._options = None
  _MERGEANALYSISREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _PUBLISHCHANGESETREQUEST.fields_by_name['repository']._options = None
  _PUBLISHCHANGESETREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _MERGEBRANCHREQUEST.fields_by_name['repository']._options = None
  _MERGEBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CENSORREQUEST.fields_by_name['repository']._options = None
  _CENSORREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _MERCURIALOPERATIONSSERVICE.methods_by_name['MergeAnalysis']._options = None
  _MERCURIALOPERATIONSSERVICE.methods_by_name['MergeAnalysis']._serialized_options = b'\372\227(\002\010\002'
  _MERCURIALOPERATIONSSERVICE.methods_by_name['PublishChangeset']._options = None
  _MERCURIALOPERATIONSSERVICE.methods_by_name['PublishChangeset']._serialized_options = b'\372\227(\002\010\001'
  _MERCURIALOPERATIONSSERVICE.methods_by_name['Censor']._options = None
  _MERCURIALOPERATIONSSERVICE.methods_by_name['Censor']._serialized_options = b'\372\227(\002\010\001'
  _MERCURIALOPERATIONSSERVICE.methods_by_name['MergeBranch']._options = None
  _MERCURIALOPERATIONSSERVICE.methods_by_name['MergeBranch']._serialized_options = b'\372\227(\002\010\001'
  _globals['_MERCURIALPERMISSIONS']._serialized_start=1649
  _globals['_MERCURIALPERMISSIONS']._serialized_end=1705
  _globals['_PRECHECKUPDATEERROR']._serialized_start=1707
  _globals['_PRECHECKUPDATEERROR']._serialized_end=1814
  _globals['_MERGEANALYSISREQUEST']._serialized_start=131
  _globals['_MERGEANALYSISREQUEST']._serialized_end=279
  _globals['_MERGEANALYSISRESPONSE']._serialized_start=282
  _globals['_MERGEANALYSISRESPONSE']._serialized_end=583
  _globals['_PUBLISHCHANGESETREQUEST']._serialized_start=586
  _globals['_PUBLISHCHANGESETREQUEST']._serialized_end=759
  _globals['_PUBLISHCHANGESETRESPONSE']._serialized_start=761
  _globals['_PUBLISHCHANGESETRESPONSE']._serialized_end=787
  _globals['_PUBLISHCHANGESETERROR']._serialized_start=789
  _globals['_PUBLISHCHANGESETERROR']._serialized_end=869
  _globals['_MERGEBRANCHREQUEST']._serialized_start=872
  _globals['_MERGEBRANCHREQUEST']._serialized_end=1161
  _globals['_MERGEBRANCHRESPONSE']._serialized_start=1163
  _globals['_MERGEBRANCHRESPONSE']._serialized_end=1238
  _globals['_MERGEBRANCHERROR']._serialized_start=1241
  _globals['_MERGEBRANCHERROR']._serialized_end=1472
  _globals['_CENSORREQUEST']._serialized_start=1475
  _globals['_CENSORREQUEST']._serialized_end=1629
  _globals['_CENSORRESPONSE']._serialized_start=1631
  _globals['_CENSORRESPONSE']._serialized_end=1647
  _globals['_MERCURIALOPERATIONSSERVICE']._serialized_start=1817
  _globals['_MERCURIALOPERATIONSSERVICE']._serialized_end=2179
# @@protoc_insertion_point(module_scope)
